import copy
import json
import os.path
import multiprocessing
from pathlib import Path
from typing import Optional, Dict, Any
from .jobs_history_journal import JobsHistory
from .db_connection_window import DatabaseConnectionWindow
from ...core.logger import get_module_logger
from ..base_window import BaseMainWindow
from PyQt6.QtWidgets import QDialog
from ..dialogs import SaveConfirmationDialog, SaveAsDialog

try:
    from PyQt6 import QtGui
    from PyQt6.QtGui import QIcon
    from PyQt6.QtGui import QAction
    from PyQt6.QtCore import pyqtSignal, pyqtSlot, Qt
    from PyQt6.QtSql import QSqlDatabase
    from PyQt6.QtWidgets import (
    QWidget, QLabel, QVBoxLayout, QHBoxLayout, QLineEdit, QScrollArea, QMessageBox,
    QSizePolicy, QToolBar, QComboBox, QFormLayout, QSpacerItem, QTextEdit,
    QMenu, QMainWindow, QApplication, QCheckBox, QPushButton, QTabWidget
    )
    pyqt_version = 6
except ImportError:
    from PyQt5 import QtGui
    from PyQt5.QtGui import QIcon
    from PyQt5.QtWidgets import QAction
    from PyQt5.QtCore import pyqtSignal, pyqtSlot, Qt
    from PyQt5.QtSql import QSqlDatabase
    from PyQt5.QtWidgets import (
    QWidget, QLabel, QVBoxLayout, QHBoxLayout, QLineEdit, QScrollArea, QMessageBox,
    QSizePolicy, QToolBar, QComboBox, QFormLayout, QSpacerItem, QTextEdit,
    QMenu, QMainWindow, QApplication, QCheckBox, QPushButton, QTabWidget
    )
    pyqt_version = 5


from ...utils import utils
from .configurer_tabs import src_tab, detector_tab, handler_tab, visualizer_tab, database_tab, tracker_tab, events_tab


class SaveWindow(QWidget):
    save_params_signal = pyqtSignal(str)

    def __init__(self):
        super().__init__()
        self.h_layout = QHBoxLayout()
        self.save_button = QPushButton('Save parameters', self)
        self.save_button.clicked.connect(self._save_data)
        self.file_name = QLabel('Enter file name')
        self.file_name_edit = QTextEdit()
        self.file_name_edit.setText('.json')
        self.file_name_edit.setFixedHeight(self.save_button.geometry().height())
        self.h_layout.addWidget(self.file_name_edit)
        self.h_layout.addWidget(self.save_button)
        self.setLayout(self.h_layout)

    @pyqtSlot()
    def _save_data(self):
        file_name = self.file_name_edit.toPlainText()
        if not file_name.strip('.json'):
            file_name = 'temp.json'
        self.save_params_signal.emit(file_name)
        self.close()


class ConfigurerMainWindow(QDialog):
    display_zones_signal = pyqtSignal(dict)
    add_zone_signal = pyqtSignal(int)
    config_changed = pyqtSignal(str)  # Сигнал изменения конфигурации
    window_closed = pyqtSignal()  # Сигнал закрытия окна

    def __init__(self, config_file_name, win_width, win_height, parent=None):
        # Инициализируем базовый класс
        super().__init__(parent)
        
        self.logger = get_module_logger("configurer_window")
        self.config_file_name = config_file_name
        self.setWindowTitle("EvilEye Configurer")
        self.resize(win_width, win_height)
        
        # Устанавливаем модальность
        self.setModal(True)
        
        # Добавляем атрибуты, которые использовались из BaseMainWindow
        self.has_unsaved_changes = False
        self.config_history_manager = None

        self.is_db_connected = False

        file_path = self.config_file_name  # 'configurer/initial_config.json'
        full_path = os.path.join(utils.get_project_root(), file_path)
        with open(full_path, 'r+') as params_file:
            config_params = json.load(params_file)
        
        # Больше не распаковываем секции пайплайна на корневой уровень

        with open(os.path.join(utils.get_project_root(), "database_config.json"), 'r+') as database_config_file:
            database_params = json.load(database_config_file)

        try:
            with open("credentials.json") as creds_file:
                self.credentials = json.load(creds_file)
        except FileNotFoundError as ex:
            self.credentials = {}

        database_creds = self.credentials.get("database", None)
        if not database_creds:
            database_creds = dict()

        try:
            with open(os.path.join(utils.get_project_root(), "database_config.json")) as data_config_file:
                self.database_config = json.load(data_config_file)
        except FileNotFoundError as ex:
            self.database_config = dict()
            self.database_config["database"] = dict()

        database_creds["user_name"] = database_creds.get("user_name", "evil_eye_user")
        database_creds["password"] = database_creds.get("password", "")
        database_creds["database_name"] = database_creds.get("database_name", "evil_eye_db")
        database_creds["host_name"] = database_creds.get("host_name", "localhost")
        database_creds["port"] = database_creds.get("port", 5432)
        database_creds["admin_user_name"] = database_creds.get("admin_user_name", "postgres")
        database_creds["admin_password"] = database_creds.get("admin_password", "")

        self.database_config["database"]["user_name"] = self.database_config["database"].get("user_name", database_creds["user_name"])
        self.database_config["database"]["password"] = self.database_config["database"].get("password", database_creds["password"])
        self.database_config["database"]["database_name"] = self.database_config["database"].get("database_name", database_creds["database_name"])
        self.database_config["database"]["host_name"] = self.database_config["database"].get("host_name", database_creds["host_name"])
        self.database_config["database"]["port"] = self.database_config["database"].get("port", database_creds["port"])
        self.database_config["database"]["admin_user_name"] = self.database_config["database"].get("admin_user_name", database_creds["admin_user_name"])
        self.database_config["database"]["admin_password"] = self.database_config["database"].get("admin_password", database_creds["admin_password"])

        self.params = config_params
        self.default_src_params = self.params['sources'][0]
        self.default_det_params = self.params['detectors'][0]
        self.default_track_params = self.params['trackers'][0]
        self.default_vis_params = self.params['visualizer']
        self.default_db_params = self.database_config['database']
        self.default_events_params = self.params['events_detectors']
        self.default_handler_params = self.params['objects_handler']
        self.config_result = copy.deepcopy(config_params)

        self.src_hist_clicked = False
        self.jobs_hist_clicked = False

        self.proj_root = utils.get_project_root()
        self.hor_layouts = {}
        self.det_button = None
        self.track_buttons = []
        self.split_check_boxes = []
        self.botsort_check_boxes = []
        self.coords_edits = []
        self.src_counter = 0
        self.jobs_history = None
        # self.db_window = DatabaseConnectionWindow(self.database_config)
        # self.db_window.database_connection_signal.connect(self._open_history)
        # self.db_window.setVisible(False)

        self.save_win = SaveWindow()
        self.save_win.save_params_signal.connect(self._save_params)

        self.tab_params = {}  # Словарь для сопоставления полей интерфейса с полями json-файла

        self._setup_tabs()

        self._create_actions()
        self._connect_actions()
        self.run_flag = False
        
        # Создаем основной layout для QDialog
        main_layout = QVBoxLayout()
        
        # Добавляем меню-кнопки
        menu_layout = self._create_menu_bar()
        main_layout.addLayout(menu_layout)
        
        # Создаем scroll area для вкладок
        self.scroll_area = QScrollArea()
        self.scroll_area.setWidgetResizable(True)
        self.scroll_area.setWidget(self.tabs)
        self.scroll_area.setHorizontalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAsNeeded)
        self.scroll_area.setVerticalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAsNeeded)
        
        # Добавляем scroll area с вкладками
        main_layout.addWidget(self.scroll_area)
        
        # Устанавливаем layout для диалога
        self.setLayout(main_layout)
        
        # Устанавливаем минимальные размеры окна
        self.setMinimumSize(800, 600)
        self.resize(1200, 800)
        
        self._connect_to_db()

        self.result_filename = None
        multiprocessing.set_start_method('spawn')
        
        # Инициализация отслеживания изменений
        self._init_change_tracking()
        
        # Подключение сигналов для отслеживания изменений
        self._connect_change_tracking_signals()

    def _setup_tabs(self):
        self.tabs = QTabWidget()
        self.tabs.setSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Expanding)
        self.tabs.addTab(src_tab.SourcesTab(self.params['sources'], self.credentials, parent=self), 'Sources')
        self.tabs.addTab(detector_tab.DetectorTab(self.params['detectors']), 'Detectors')
        self.tabs.addTab(tracker_tab.TrackerTab(self.params['trackers']), 'Trackers')
        self.tabs.addTab(handler_tab.HandlerTab(self.params, self.database_config), 'Objects handler')
        self.tabs.addTab(database_tab.DatabaseTab(self.params, self.database_config), 'Database')
        self.tabs.addTab(visualizer_tab.VisualizerTab(self.params), 'Visualizer')
        self.tabs.addTab(events_tab.EventsTab(self.params), 'Events')
        self.sections = ['sources', 'detectors', 'trackers', 'objects_handler',
                         'database', 'visualizer', 'events_detectors']

        source_tab = self.tabs.widget(0)
        source_tab.connection_win_signal.connect(self._check_db_connection)
        det_tab = self.tabs.widget(1)
        track_tab = self.tabs.widget(2)
        det_tab.tracker_enabled_signal.connect(track_tab.enable_add_tracker_button)

        self.tab_params['sources'] = self.tabs.widget(0)
        self.tab_params['detectors'] = self.tabs.widget(1)
        self.tab_params['trackers'] = self.tabs.widget(2)
        self.tab_params['objects_handler'] = self.tabs.widget(3)
        self.tab_params['database'] = self.tabs.widget(4)
        self.tab_params['visualizer'] = self.tabs.widget(5)
        self.tab_params['events_detectors'] = self.tabs.widget(6)

    def _create_menu_bar(self):
        # Для QDialog создаем меню как обычные кнопки в layout
        # Вместо menuBar создаем горизонтальный layout с кнопками
        menu_layout = QHBoxLayout()
        
        # Создаем кнопки вместо меню
        self.open_history_btn = QPushButton('Open History', self)
        self.load_from_history_btn = QPushButton('Load from History', self)
        self.save_btn = QPushButton('Save', self)
        self.save_as_btn = QPushButton('Save As', self)
        self.run_btn = QPushButton('Run', self)
        
        # Подключаем сигналы
        self.open_history_btn.clicked.connect(self._open_history)
        self.load_from_history_btn.clicked.connect(self._load_from_history)
        self.save_btn.clicked.connect(self._save_config)
        self.save_as_btn.clicked.connect(self._save_config_as)
        self.run_btn.clicked.connect(self._run_app)
        
        # Добавляем кнопки в layout
        menu_layout.addWidget(self.open_history_btn)
        menu_layout.addWidget(self.load_from_history_btn)
        menu_layout.addWidget(self.save_btn)
        menu_layout.addWidget(self.save_as_btn)
        menu_layout.addWidget(self.run_btn)
        menu_layout.addStretch()
        
        return menu_layout


    def _create_actions(self):  # Создание кнопок-действий
        self.save_params = QAction('&Save parameters', self)
        self.save_params.setIcon(QIcon(os.path.join(utils.get_project_root(), 'icons', 'save_icon.svg')))
        self.save_as_params = QAction('&Save As...', self)
        self.save_as_params.setIcon(QIcon(os.path.join(utils.get_project_root(), 'icons', 'save_icon.svg')))
        self.open_jobs_history = QAction('&Open history', self)
        self.load_from_history = QAction('&Load from History', self)
        self.start_app = QAction('&Run app', self)
        self.start_app.setIcon(QIcon(os.path.join(utils.get_project_root(), 'icons', 'run_app.svg')))
        icon_path = os.path.join(utils.get_project_root(), 'icons', 'journal.svg')
        self.open_jobs_history.setIcon(QIcon(icon_path))
        self.load_from_history.setIcon(QIcon(icon_path))

    def _connect_actions(self):
        self.save_params.triggered.connect(self._save_config)
        self.save_as_params.triggered.connect(self._save_config_as)
        self.open_jobs_history.triggered.connect(self._check_db_connection)
        self.load_from_history.triggered.connect(self._load_from_history)
        self.start_app.triggered.connect(self._prepare_running)

    @pyqtSlot()
    def _prepare_running(self):
        self.run_flag = True
        # Сохраняем конфигурацию перед запуском
        self._save_config()

    def _run_app(self):
        import subprocess
        import sys
        from pathlib import Path
        
        # Получаем путь к process.py
        project_root = Path(__file__).parent.parent.parent.parent
        process_script = project_root / "evileye" / "process.py"
        
        if process_script.exists():
            cmd = [sys.executable, str(process_script), "--config", self.result_filename, "--gui"]
            self.new_process = subprocess.Popen(cmd, cwd=project_root)
        else:
            self.logger.error(f"process.py not found at {process_script}")

    @pyqtSlot()
    def _open_save_win(self):
        self.save_win.show()

    @pyqtSlot(str)
    def _save_params(self, file_name):
        self._process_params_strings()
        self.result_filename = os.path.join(utils.get_project_root(), file_name)
        with open(self.result_filename, 'w') as file:
            json.dump(self.config_result, file, indent=4)

        if self.run_flag:
            self.save_win.close()
            self._run_app()

    @pyqtSlot()
    def _check_db_connection(self):
        sender = self.sender()
        if isinstance(sender, QAction):
            self.jobs_hist_clicked = True
            self.src_hist_clicked = False
            if self.is_db_connected:
                self._open_history()
            else:
                self._connect_to_db()
                # if self.db_window.isVisible():
                #     self.db_window.setVisible(False)
                # else:
                #     self.db_window.setVisible(True)
        else:
            self.src_hist_clicked = True
            self.jobs_hist_clicked = False
            if self.is_db_connected:
                self.tabs.widget(0).open_src_list()
            else:
                self._connect_to_db()
                # if self.db_window.isVisible():
                #     self.db_window.setVisible(False)
                # else:
                #     self.db_window.setVisible(True)

    @pyqtSlot()
    def _open_history(self):
        if self.jobs_hist_clicked:
            if not self.jobs_history:
                self.jobs_history = JobsHistory()
                self.jobs_history.setVisible(False)
                
                # Интегрируем с ConfigHistoryManager, если доступен
                if hasattr(self, 'db_controller') and self.db_controller:
                    try:
                        from ...database.config_history_manager import ConfigHistoryManager
                        config_history_manager = ConfigHistoryManager(self.db_controller)
                        self.jobs_history.set_config_history_manager(config_history_manager)
                        self.logger.info("ConfigHistoryManager integrated with JobsHistory")
                    except Exception as e:
                        self.logger.warning(f"Could not integrate ConfigHistoryManager: {e}")

            if self.jobs_history.isVisible():
                self.jobs_history.setVisible(False)
            else:
                self.jobs_history.setVisible(True)

        if self.src_hist_clicked:
            self.tabs.widget(0).open_src_list()

    @pyqtSlot()
    def _load_from_history(self):
        """Загрузить конфигурацию из истории"""
        try:
            # Проверяем доступность базы данных
            if not hasattr(self, 'db_controller') or not self.db_controller:
                QMessageBox.warning(
                    self,
                    "База данных недоступна",
                    "Для загрузки конфигурации из истории необходимо подключение к базе данных."
                )
                return
            
            # Создаем ConfigHistoryManager
            from ...database.config_history_manager import ConfigHistoryManager
            config_history_manager = ConfigHistoryManager(self.db_controller)
            
            # Получаем список уникальных конфигураций
            unique_configs = config_history_manager.get_unique_configurations(limit=20)
            
            if not unique_configs:
                QMessageBox.information(
                    self,
                    "История пуста",
                    "В истории конфигураций нет записей.\n"
                    "Запустите приложение хотя бы один раз для создания истории."
                )
                return
            
            # Создаем диалог выбора конфигурации
            from ...dialogs import ConfigRestoreDialog
            
            # Показываем диалог для выбора конфигурации
            # Используем первую конфигурацию как пример
            selected_config = unique_configs[0]
            
            # Создаем диалог восстановления
            dialog = ConfigRestoreDialog(selected_config, self)
            dialog.config_restored.connect(self._on_config_loaded_from_history)
            dialog.exec()
            
        except Exception as e:
            self.logger.error(f"Error loading from history: {e}")
            QMessageBox.critical(
                self,
                "Ошибка",
                f"Не удалось загрузить конфигурацию из истории:\n{str(e)}"
            )
    
    @pyqtSlot(dict)
    def _on_config_loaded_from_history(self, restore_data: dict):
        """Обработчик загрузки конфигурации из истории"""
        try:
            config_info = restore_data.get('config_info', {})
            configuration_data = config_info.get('configuration_info', {})
            
            if not configuration_data:
                QMessageBox.warning(
                    self,
                    "Ошибка",
                    "Не удалось получить данные конфигурации из истории."
                )
                return
            
            # Загружаем конфигурацию в текущие вкладки
            self._load_configuration_data(configuration_data)
            
            # Показываем уведомление об успешной загрузке
            QMessageBox.information(
                self,
                "Конфигурация загружена",
                f"Конфигурация из истории успешно загружена.\n"
                f"Job ID: {restore_data.get('job_id', 'N/A')}\n"
                f"Дата создания: {config_info.get('creation_time', 'N/A')}\n\n"
                f"Не забудьте сохранить изменения, если они нужны."
            )
            
            # Отмечаем, что есть несохраненные изменения
            self.has_unsaved_changes = True
            self._update_window_title()
            
        except Exception as e:
            self.logger.error(f"Error processing loaded config: {e}")
            QMessageBox.critical(
                self,
                "Ошибка",
                f"Не удалось обработать загруженную конфигурацию:\n{str(e)}"
            )
    
    def _load_configuration_data(self, config_data: dict):
        """Загрузить данные конфигурации в вкладки"""
        try:
            # Загружаем данные в соответствующие вкладки
            if 'sources' in config_data:
                self.tab_params['sources'].set_params(config_data['sources'])
            
            if 'detectors' in config_data:
                self.tab_params['detectors'].set_params(config_data['detectors'])
            
            if 'trackers' in config_data:
                self.tab_params['trackers'].set_params(config_data['trackers'])
            
            if 'handlers' in config_data:
                self.tab_params['handlers'].set_params(config_data['handlers'])
            
            if 'visualizers' in config_data:
                self.tab_params['visualizers'].set_params(config_data['visualizers'])
            
            if 'events' in config_data:
                self.tab_params['events'].set_params(config_data['events'])
            
            if 'events_detectors' in config_data:
                self.tab_params['events_detectors'].set_params(config_data['events_detectors'])
            
            self.logger.info("Configuration data loaded into tabs")
            
        except Exception as e:
            self.logger.error(f"Error loading configuration data: {e}")
            raise

    def _process_params_strings(self):
        configs = []
        src_config = self.tab_params['sources'].get_params()
        configs.append(('sources', src_config))
        det_config = self.tab_params['detectors'].get_params()
        configs.append(('detectors', det_config))
        track_config = self.tab_params['trackers'].get_params()
        configs.append(('trackers', track_config))
        vis_config = self.tab_params['visualizer'].get_params()
        configs.append(('visualizer', vis_config))
        handler_config = self.tab_params['objects_handler'].get_params()
        configs.append(('objects_handler', handler_config))
        events_config = self.tab_params['events_detectors'].get_params()
        configs.append(('events_detectors', events_config))
        self._create_resulting_config(configs, self.params)

    def _create_resulting_config(self, configs, default_config):
        # Секции, которые относятся к пайплайну
        pipeline_sections = {
            'sources', 'preprocessors', 'detectors', 'trackers', 'mc_trackers',
            'attributes_roi', 'attributes_classifier'
        }

        # Обеспечим наличие секции pipeline
        if 'pipeline' not in self.config_result or not isinstance(self.config_result['pipeline'], dict):
            self.config_result['pipeline'] = {}

        # Сначала очистим возможные дубли секций пайплайна на корневом уровне
        for key in list(self.config_result.keys()):
            if key in pipeline_sections:
                try:
                    del self.config_result[key]
                except Exception:
                    pass

        # Запишем актуальные параметры в нужные разделы
        for section_config in configs:
            section_name = section_config[0]
            section_params = section_config[1]
            if section_name in pipeline_sections:
                self.config_result['pipeline'][section_name] = section_params
            else:
                self.config_result[section_name] = section_params

    def _connect_to_db(self):
        db_params = self.database_config['database']
        db = QSqlDatabase.addDatabase("QPSQL", 'jobs_conn')
        db.setHostName(db_params['host_name'])
        db.setDatabaseName(db_params['database_name'])
        db.setUserName(db_params['user_name'])
        db.setPassword(db_params['password'])
        db.setPort(db_params['port'])
        if not db.open():
            QMessageBox.critical(
                None,
                "Connection error",
                str(db.lastError().text()),
            )
            self.is_db_connected = False
        else:
            self.is_db_connected = True

    def closeEvent(self, event):
        # Проверяем, есть ли несохраненные изменения
        if self.has_unsaved_changes:
            reply = QMessageBox.question(
                self, 
                'Несохраненные изменения',
                'У вас есть несохраненные изменения. Хотите сохранить их перед закрытием?',
                QMessageBox.StandardButton.Save | 
                QMessageBox.StandardButton.Discard | 
                QMessageBox.StandardButton.Cancel,
                QMessageBox.StandardButton.Save
            )
            
            if reply == QMessageBox.StandardButton.Cancel:
                event.ignore()
                return
            elif reply == QMessageBox.StandardButton.Save:
                # Сохраняем конфигурацию
                self._save_config()
        
        # Закрываем вкладки
        for tab_idx in range(self.tabs.count()):
            tab = self.tabs.widget(tab_idx)
            tab.close()
        
        # Очищаем подключение к базе данных
        self.logger.info('DB jobs_conn removed')
        QSqlDatabase.removeDatabase('jobs_conn')
        
        # Испускаем сигнал закрытия окна
        self.window_closed.emit()
        
        # НЕ закрываем все окна приложения!
        event.accept()

    def resizeEvent(self, event: QtGui.QResizeEvent) -> None:
        super().resizeEvent(event)
    
    # === Новые методы для отслеживания изменений и улучшенного сохранения ===
    
    def _init_change_tracking(self) -> None:
        """Инициализация отслеживания изменений"""
        self._original_config = copy.deepcopy(self.config_result)
        self._has_unsaved_changes = False
        
        # Сохраняем оригинальные значения для сравнения
        self._original_tab_data = {}
        for tab_name, tab_widget in self.tab_params.items():
            if hasattr(tab_widget, 'get_params'):
                self._original_tab_data[tab_name] = tab_widget.get_params()
    
    def _connect_change_tracking_signals(self) -> None:
        """Подключение сигналов для отслеживания изменений"""
        # Подключаем сигналы изменения вкладок
        for tab_name, tab_widget in self.tab_params.items():
            if hasattr(tab_widget, 'params_changed'):
                tab_widget.params_changed.connect(self._on_tab_params_changed)
            elif hasattr(tab_widget, 'valueChanged'):
                # Для виджетов с сигналом valueChanged
                tab_widget.valueChanged.connect(self._on_tab_params_changed)
            elif hasattr(tab_widget, 'textChanged'):
                # Для текстовых полей
                tab_widget.textChanged.connect(self._on_tab_params_changed)
    
    @pyqtSlot()
    def _on_tab_params_changed(self) -> None:
        """Обработчик изменения параметров вкладки"""
        self._check_for_changes()
    
    def _check_for_changes(self) -> None:
        """Проверка наличия изменений в конфигурации"""
        try:
            # Получаем текущие параметры
            current_config = self._get_current_config()
            
            # Сравниваем с оригинальной конфигурацией
            has_changes = current_config != self._original_config
            
            if has_changes != self._has_unsaved_changes:
                self._has_unsaved_changes = has_changes
                self.has_unsaved_changes = has_changes
                
                if has_changes:
                    self.logger.debug("Configuration changes detected")
                else:
                    self.logger.debug("Configuration changes resolved")
                    
        except Exception as e:
            self.logger.error(f"Error checking for changes: {e}")
    
    def _get_current_config(self) -> Dict[str, Any]:
        """Получить текущую конфигурацию из всех вкладок"""
        try:
            self._process_params_strings()
            return copy.deepcopy(self.config_result)
        except Exception as e:
            self.logger.error(f"Error getting current config: {e}")
            return {}
    
    @pyqtSlot()
    def _save_config(self) -> None:
        """Сохранить конфигурацию в текущий файл"""
        if not self.config_file_name:
            # Если нет текущего файла, показываем диалог "Сохранить как"
            self._save_config_as()
            return
        
        try:
            # Получаем текущую конфигурацию
            current_config = self._get_current_config()
            
            # Валидируем конфигурацию
            if not self._validate_config(current_config):
                return
            
            # Путь сохранения
            full_path = os.path.join(utils.get_project_root(), self.config_file_name)

            # Централизованное сохранение через контроллер, если доступен
            controller = getattr(getattr(self, 'parent', None), 'controller', None)
            if controller and hasattr(controller, 'save_config'):
                try:
                    # Передаём путь в контроллер
                    controller.params_path = full_path
                    # Обновляем текущие параметры контроллера перед сохранением
                    controller.params = copy.deepcopy(current_config)
                    ok = controller.save_config(full_path)
                    if not ok:
                        raise RuntimeError("controller.save_config() returned False")
                except Exception as e:
                    # Fallback: локальная запись, если контроллер не сохранил
                    with open(full_path, 'w', encoding='utf-8') as file:
                        json.dump(current_config, file, indent=4, ensure_ascii=False)
            else:
                # Fallback: локальная запись, если контроллер недоступен
                with open(full_path, 'w', encoding='utf-8') as file:
                    json.dump(current_config, file, indent=4, ensure_ascii=False)
            
            # Обновляем оригинальную конфигурацию
            self._original_config = copy.deepcopy(current_config)
            self._has_unsaved_changes = False
            self.has_unsaved_changes = False
            
            # Показываем уведомление об успешном сохранении
            QMessageBox.information(
                self, 
                "Сохранение", 
                f"Конфигурация успешно сохранена в:\n{full_path}"
            )
            
            # Испускаем сигнал об изменении конфигурации
            self.config_changed.emit(self.config_file_name)
            
            self.logger.info(f"Configuration saved to: {full_path}")
            
        except Exception as e:
            self.logger.error(f"Error saving configuration: {e}")
            QMessageBox.critical(
                self, 
                "Ошибка сохранения", 
                f"Не удалось сохранить конфигурацию:\n{str(e)}"
            )
    
    @pyqtSlot()
    def _save_config_as(self) -> None:
        """Сохранить конфигурацию под новым именем"""
        try:
            # Показываем диалог выбора файла
            file_path = SaveAsDialog.show_dialog(self.config_file_name, self)
            
            if not file_path:
                return  # Пользователь отменил
            
            # Получаем текущую конфигурацию
            current_config = self._get_current_config()
            
            # Валидируем конфигурацию
            if not self._validate_config(current_config):
                return
            
            # Централизованное сохранение через контроллер, если доступен
            controller = getattr(getattr(self, 'parent', None), 'controller', None)
            if controller and hasattr(controller, 'save_config'):
                try:
                    controller.params_path = file_path
                    controller.params = copy.deepcopy(current_config)
                    ok = controller.save_config(file_path)
                    if not ok:
                        raise RuntimeError("controller.save_config() returned False")
                except Exception:
                    # Fallback: локальная запись
                    with open(file_path, 'w', encoding='utf-8') as file:
                        json.dump(current_config, file, indent=4, ensure_ascii=False)
            else:
                # Fallback: локальная запись
                with open(file_path, 'w', encoding='utf-8') as file:
                    json.dump(current_config, file, indent=4, ensure_ascii=False)
            
            # Обновляем текущий файл конфигурации
            self.config_file_name = file_path
            self.config_file = file_path
            
            # Обновляем оригинальную конфигурацию
            self._original_config = copy.deepcopy(current_config)
            self._has_unsaved_changes = False
            self.has_unsaved_changes = False
            
            # Обновляем заголовок окна
            self.setWindowTitle(f"EvilEye Configurer - {Path(file_path).name}")
            
            # Показываем уведомление об успешном сохранении
            QMessageBox.information(
                self, 
                "Сохранение", 
                f"Конфигурация успешно сохранена в:\n{file_path}"
            )
            
            # Испускаем сигнал об изменении конфигурации
            self.config_changed.emit(self.config_file_name)
            
            self.logger.info(f"Configuration saved as: {file_path}")
            
        except Exception as e:
            self.logger.error(f"Error saving configuration as: {e}")
            QMessageBox.critical(
                self, 
                "Ошибка сохранения", 
                f"Не удалось сохранить конфигурацию:\n{str(e)}"
            )
    
    def _validate_config(self, config: Dict[str, Any]) -> bool:
        """Валидация конфигурации перед сохранением"""
        try:
            # Проверяем обязательные секции
            required_sections = ['sources', 'detectors', 'trackers', 'visualizer']
            for section in required_sections:
                if section not in config:
                    QMessageBox.warning(
                        self, 
                        "Ошибка валидации", 
                        f"Отсутствует обязательная секция: {section}"
                    )
                    return False
            
            # Проверяем источники
            sources = config.get('sources', [])
            if not sources:
                QMessageBox.warning(
                    self, 
                    "Ошибка валидации", 
                    "Необходимо настроить хотя бы один источник"
                )
                return False
            
            # Дополнительные проверки можно добавить здесь
            
            return True
            
        except Exception as e:
            self.logger.error(f"Error validating config: {e}")
            QMessageBox.critical(
                self, 
                "Ошибка валидации", 
                f"Ошибка при валидации конфигурации:\n{str(e)}"
            )
            return False
    
    # === Реализация абстрактных методов BaseMainWindow ===
    
    def get_config_data(self) -> Optional[Dict[str, Any]]:
        """Получить данные конфигурации для сохранения"""
        try:
            return self._get_current_config()
        except Exception as e:
            self.logger.error(f"Error getting config data: {e}")
            return None
    
    def apply_config_data(self, config_data: Dict[str, Any]) -> bool:
        """Применить данные конфигурации"""
        try:
            # Обновляем параметры
            self.params = copy.deepcopy(config_data)
            self.config_result = copy.deepcopy(config_data)
            
            # Обновляем вкладки
            for tab_name, tab_widget in self.tab_params.items():
                if hasattr(tab_widget, 'set_params'):
                    section_data = config_data.get(tab_name, {})
                    tab_widget.set_params(section_data)
            
            # Обновляем оригинальную конфигурацию
            self._original_config = copy.deepcopy(config_data)
            self._has_unsaved_changes = False
            self.has_unsaved_changes = False
            
            self.logger.info("Configuration data applied successfully")
            return True
            
        except Exception as e:
            self.logger.error(f"Error applying config data: {e}")
            return False
    
    def on_config_changed(self, config_file: str) -> None:
        """Обработчик изменения конфигурации"""
        if config_file == self.config_file_name:
            self.logger.info(f"Configuration file changed: {config_file}")
            # Можно добавить логику для перезагрузки конфигурации



try:
    from PyQt6.QtCore import QThread, pyqtSignal, QEventLoop, QTimer
    from PyQt6 import QtGui
    from PyQt6.QtCore import Qt
    from PyQt6.QtGui import QPixmap
    pyqt_version = 6
except ImportError:
    from PyQt5.QtCore import QThread, pyqtSignal, QEventLoop, QTimer
    from PyQt5 import QtGui
    from PyQt5.QtCore import Qt
    from PyQt5.QtGui import QPixmap
    pyqt_version = 5

from timeit import default_timer as timer
from utils import utils
from queue import Queue
import time
import cv2
import logging


class TableDataThread(QThread):
    append_record_signal = pyqtSignal(list)
    update_record_signal = pyqtSignal(list)

    def __init__(self, fps, db_controller, logger_name: str | None = None, parent_logger: logging.Logger | None = None):
        super().__init__()
        base_name = "evileye.table_data_thread"
        full_name = f"{base_name}.{logger_name}" if logger_name else base_name
        self.logger = parent_logger or logging.getLogger(full_name)

        self.queue = Queue()
        self.db_controller = db_controller
        self.fps = fps

        self.run_flag = False

    def append_data(self, data):
        self.queue.put(data)

    def start_thread(self):
        self.run_flag = True
        self.start()

    def stop_thread(self):
        self.run_flag = False
        self.logger.info('Data preparation thread stopped')

    def run(self):
        while self.run_flag:
            elapsed_seconds = self.process_query()
            sleep_seconds = 1. / self.fps - elapsed_seconds
            if sleep_seconds > 0.0:
                time.sleep(sleep_seconds)
            else:
                time.sleep(0.01)

    def process_query(self):
        try:
            query_type, query_string, data = self.queue.get()
        except ValueError:
            return 0
        begin_it = timer()
        # self.logger.debug(data)
        records = self.db_controller.query(query_string, data)
        end_it = timer()
        elapsed_seconds = end_it - begin_it
        # self.logger.debug(records)
        if query_type == 'Insert':
            self.append_record_signal.emit(records)
        elif query_type == 'Update':
            self.update_record_signal.emit(records)
        return elapsed_seconds




"""
GUI Manager for EvilEye application.

Manages GUI components lifecycle and separates core initialization from GUI initialization.
"""

import sys
import os
from pathlib import Path
from typing import Optional, Dict, Any
import logging

try:
    from PyQt6.QtWidgets import QApplication
    from PyQt6.QtCore import QTimer, QThread, pyqtSignal, QObject
    pyqt_version = 6
except ImportError:
    from PyQt5.QtWidgets import QApplication
    from PyQt5.QtCore import QTimer, QThread, pyqtSignal, QObject
    pyqt_version = 5

from .gui_mode import GUIMode
from .interfaces import IVisualizationProvider, IProgressReporter
from ..core.logger import get_module_logger


class GUIManager(QObject):
    """Manages GUI components and their lifecycle."""
    
    # Signals for GUI events
    gui_ready = pyqtSignal()  # Emitted when GUI is ready
    gui_shown = pyqtSignal()  # Emitted when GUI is shown
    gui_hidden = pyqtSignal()  # Emitted when GUI is hidden
    
    def __init__(self, mode: GUIMode, config: Dict[str, Any], logger: Optional[logging.Logger] = None):
        """
        Initialize GUI Manager.
        
        Args:
            mode: GUI operation mode
            config: Configuration dictionary
            logger: Optional logger instance
        """
        super().__init__()
        # Initialize all long-lived variables in constructor
        self.mode = mode
        self.config = config
        self.logger = logger or get_module_logger("gui_manager")
        
        self.qt_app: Optional[QApplication] = None
        self.main_window = None
        self.visualization_adapter = None
        self.progress_reporter = None
        
        # Flag monitoring for hidden GUI mode
        self.flag_monitor_timer: Optional[QTimer] = None
        # GUI activation flag file path - initialize in constructor
        flag_file_config = config.get("controller", {}).get("gui_flag_file", "EvilEyeData/.show_gui")
        self.flag_file_path: Optional[Path] = Path(flag_file_config)
        
    def create_gui_application(self) -> QApplication:
        """
        Create QApplication instance.
        
        Returns:
            QApplication instance
        """
        if self.qt_app is not None:
            return self.qt_app
            
        self.logger.info("Creating QApplication...")
        # Mitigate Qt↔GLib dispatcher conflicts
        try:
            os.environ.setdefault("QT_NO_GLIB", "1")
        except Exception:
            pass
            
        self.qt_app = QApplication.instance() or QApplication(sys.argv)
        self.logger.info("QApplication created")
        return self.qt_app
    
    def initialize(self, controller) -> None:
        """
        Initialize GUI components based on mode.
        
        Args:
            controller: Controller instance
        """
        if self.mode == GUIMode.HEADLESS:
            self.logger.info("Headless mode: skipping GUI initialization")
            return
        
        # Create QApplication
        self.create_gui_application()
        
        if self.mode == GUIMode.HIDDEN:
            self.logger.info("Hidden GUI mode: creating GUI components but not showing")
            self._create_gui_components(controller)
            self._start_flag_monitor()
        elif self.mode == GUIMode.VISIBLE:
            self.logger.info("Visible GUI mode: creating and showing GUI components")
            self._create_gui_components(controller)
            self.show_gui()
    
    def _create_gui_components(self, controller) -> None:
        """
        Create GUI components asynchronously.
        
        Args:
            controller: Controller instance
        """
        # GUI components are created in _connect_gui_to_controller()
        # This method is kept for future use if needed
        self.logger.info("GUI components will be created when connecting to controller")
        pass
    
    def show_gui(self) -> None:
        """Show GUI windows."""
        if self.main_window:
            self.main_window.show()
            self.main_window.raise_()
            self.main_window.activateWindow()
            self.gui_shown.emit()
            self.logger.info("GUI shown")
    
    def hide_gui(self) -> None:
        """Hide GUI windows."""
        if self.main_window:
            self.main_window.hide()
            self.gui_hidden.emit()
            self.logger.info("GUI hidden")
    
    def _start_flag_monitor(self) -> None:
        """Start monitoring file flag for GUI activation."""
        if self.flag_monitor_timer is not None:
            return
            
        self.logger.info(f"Starting flag monitor for file: {self.flag_file_path}")
        self.flag_monitor_timer = QTimer()
        self.flag_monitor_timer.timeout.connect(self._check_gui_activation_flag)
        self.flag_monitor_timer.start(1000)  # Check every second
    
    def _check_gui_activation_flag(self) -> None:
        """Check if GUI activation flag file exists."""
        if self.flag_file_path and self.flag_file_path.exists():
            self.logger.info("GUI activation flag detected, showing GUI")
            self.show_gui()
            # Remove flag file after activation
            try:
                self.flag_file_path.unlink()
            except Exception as e:
                self.logger.warning(f"Failed to remove flag file: {e}")
    
    def set_progress_reporter(self, reporter: IProgressReporter) -> None:
        """Set progress reporter."""
        self.progress_reporter = reporter
    
    def set_visualization_adapter(self, adapter: IVisualizationProvider) -> None:
        """Set visualization adapter."""
        self.visualization_adapter = adapter
    
    def get_qt_application(self) -> Optional[QApplication]:
        """Get QApplication instance."""
        return self.qt_app
    
    def cleanup(self) -> None:
        """Cleanup GUI resources."""
        if self.flag_monitor_timer:
            self.flag_monitor_timer.stop()
            self.flag_monitor_timer = None
        
        if self.main_window:
            try:
                self.main_window.close()
            except Exception:
                pass
            self.main_window = None



import os
import json
import datetime
from typing import Dict, List

try:
    from PyQt6.QtCore import Qt, pyqtSlot
    from PyQt6.QtWidgets import (
        QWidget, QLabel, QVBoxLayout, QHBoxLayout, QPushButton,
        QHeaderView, QComboBox, QTableWidget, QTableWidgetItem, QFileDialog, QStyledItemDelegate
    )
    from PyQt6.QtGui import QPixmap, QPainter, QPen, QColor, QBrush, QPolygonF
    from PyQt6.QtCore import QSize, QTimer, QPointF
    pyqt_version = 6
except ImportError:
    from PyQt5.QtCore import Qt, pyqtSlot
    from PyQt5.QtWidgets import (
        QWidget, QLabel, QVBoxLayout, QHBoxLayout, QPushButton,
        QHeaderView, QComboBox, QTableWidget, QTableWidgetItem, QFileDialog, QStyledItemDelegate
    )
    from PyQt5.QtGui import QPixmap, QPainter, QPen, QColor, QBrush, QPolygonF
    from PyQt5.QtCore import QSize, QTimer, QPointF
    pyqt_version = 5

from .journal_data_source_json import JsonLabelJournalDataSource
from ..core.logger import get_module_logger
import logging


class ImageDelegate(QStyledItemDelegate):
    def __init__(self, parent=None, base_dir=None, logger_name: str | None = None, parent_logger: logging.Logger | None = None):
        super().__init__(parent)
        base_name = "evileye.image_delegate"
        full_name = f"{base_name}.{logger_name}" if logger_name else base_name
        self.logger = parent_logger or logging.getLogger(full_name)
        self.base_dir = base_dir
        self.preview_width = 300
        self.preview_height = 150

    def paint(self, painter, option, index):
        if not index.isValid():
            return
            
        # Get event data from the row
        table = self.parent()
        if not table:
            return
            
        row = index.row()
        if row >= table.rowCount():
            return
            
        # Get image filename from the row (Preview or Lost preview column)
        img_filename_item = table.item(row, index.column())  # Use current column
        
        if not img_filename_item:
            return
            
        img_path = img_filename_item.text()
        
        # If no image path, just return (empty cell)
        if not img_path:
            return
        
        # Use image path directly from JSON
        if not img_path:
            return
            
        if not os.path.exists(img_path):
            # Debug: print missing image path
            self.logger.warning(f"Image not found: {img_path}")
            return
            
        # Load image
        pixmap = QPixmap(img_path)
        if pixmap.isNull():
            return

        # Compute target rect: STRETCH to full cell (match other tables)
        cell_rect = option.rect
        draw_x = cell_rect.x()
        draw_y = cell_rect.y()
        draw_w = cell_rect.width()
        draw_h = cell_rect.height()

        # Draw image stretched to cell rect
        painter.drawPixmap(cell_rect, pixmap)

        # Try to draw overlay box/zone for preview using target_rect as base
        ev_item = table.item(row, 5) if index.column() == 5 else table.item(row, 6)
        ev = ev_item.data(Qt.ItemDataRole.UserRole) if ev_item else None
        
        if ev:
            # Box - handle both dict and list formats
            box = ev.get('bounding_box') or ev.get('box')
            if box:
                painter.setPen(QPen(QColor(0, 255, 0), 2))  # Green for bbox

                # Extract corners
                if isinstance(box, dict):
                    x1 = box.get('x', 0)
                    y1 = box.get('y', 0)
                    w = box.get('width', 0)
                    h = box.get('height', 0)
                    x2 = x1 + w
                    y2 = y1 + h
                elif isinstance(box, (list, tuple)) and len(box) == 4:
                    x1, y1, x2, y2 = box
                else:
                    return

                # Normalize using full frame size when available
                if x1 > 1 or y1 > 1 or x2 > 1 or y2 > 1:
                    # Try to resolve corresponding full frame path
                    frame_path = None
                    try:
                        # Current cell path
                        img_filename_item = table.item(row, index.column())
                        cur_path = img_filename_item.text() if img_filename_item else ''
                        candidates = []
                        if cur_path:
                            # New structure: FoundPreviews/FoundFrames/LostPreviews/LostFrames
                            candidates.append(cur_path.replace('FoundPreviews', 'FoundFrames').replace('_preview.', '_frame.'))
                            candidates.append(cur_path.replace('LostPreviews', 'LostFrames').replace('_preview.', '_frame.'))
                            # Legacy support
                            candidates.append(cur_path.replace('previews', 'frames').replace('_preview.', '_frame.'))
                            candidates.append(cur_path.replace('/found_previews/', '/found_frames/').replace('_preview.', '_frame.'))
                            candidates.append(cur_path.replace('/lost_previews/', '/lost_frames/').replace('_preview.', '_frame.'))
                        # Also try constructing from event date_folder
                        ev = table.item(row, 5).data(Qt.ItemDataRole.UserRole) if index.column() == 5 else table.item(row, 6).data(Qt.ItemDataRole.UserRole)
                        if ev:
                            date_folder = ev.get('date_folder', '')
                            base_name = os.path.basename(cur_path).replace('_preview.', '_frame.')
                            # New structure: Events/YYYY-MM-DD/Images/FoundFrames or LostFrames
                            candidates.append(os.path.join(self.base_dir, 'Events', date_folder, 'Images', 'FoundFrames', base_name))
                            candidates.append(os.path.join(self.base_dir, 'Events', date_folder, 'Images', 'LostFrames', base_name))
                            # Legacy support
                            candidates.append(os.path.join(self.base_dir, 'images', date_folder, 'found_frames', base_name))
                            candidates.append(os.path.join(self.base_dir, 'images', date_folder, 'lost_frames', base_name))
                        for cand in candidates:
                            if cand and os.path.exists(cand):
                                frame_path = cand
                                break
                    except Exception:
                        frame_path = None

                    if frame_path:
                        fpix = QPixmap(frame_path)
                        if not fpix.isNull() and fpix.width() > 0 and fpix.height() > 0:
                            x1_norm = x1 / fpix.width()
                            y1_norm = y1 / fpix.height()
                            x2_norm = x2 / fpix.width()
                            y2_norm = y2 / fpix.height()
                        else:
                            # Fallback to preview size
                            x1_norm = x1 / pixmap.width()
                            y1_norm = y1 / pixmap.height()
                            x2_norm = x2 / pixmap.width()
                            y2_norm = y2 / pixmap.height()
                    else:
                        # Fallback to preview size
                        x1_norm = x1 / pixmap.width()
                        y1_norm = y1 / pixmap.height()
                        x2_norm = x2 / pixmap.width()
                        y2_norm = y2 / pixmap.height()
                else:
                    # Already normalized
                    x1_norm, y1_norm, x2_norm, y2_norm = x1, y1, x2, y2
                
                # Scale to draw area (full cell stretch)
                x = draw_x + int(x1_norm * draw_w)
                y = draw_y + int(y1_norm * draw_h)
                w = int((x2_norm - x1_norm) * draw_w)
                h = int((y2_norm - y1_norm) * draw_h)
                
                painter.drawRect(x, y, w, h)
            # Zone - use same logic as DB journal
            zc = ev.get('zone_coords')
            if zc and isinstance(zc, (list, tuple)):
                painter.setPen(QPen(QColor(255, 0, 0), 2))  # Red for zone
                painter.setBrush(QBrush(QColor(255, 0, 0, 64)))  # Semi-transparent red fill
                polygon = QPolygonF()
                for pt in zc:
                    if isinstance(pt, (list, tuple)) and len(pt) == 2:
                        px, py = pt
                        # Scale to full cell area
                        x = draw_x + int(px * draw_w)
                        y = draw_y + int(py * draw_h)
                        polygon.append(QPointF(x, y))
                painter.drawPolygon(polygon)

    def sizeHint(self, option, index):
        return QSize(self.preview_width, self.preview_height)


class DateTimeDelegate(QStyledItemDelegate):
    def __init__(self, parent=None):
        super().__init__(parent)

    def displayText(self, value, locale) -> str:
        """Format datetime to show only seconds precision"""
        try:
            if isinstance(value, str):
                # Parse ISO format datetime string
                if 'T' in value:
                    # ISO format: 2025-09-01T17:30:45.123456
                    dt = datetime.datetime.fromisoformat(value.replace('Z', '+00:00'))
                    return dt.strftime('%Y-%m-%d %H:%M:%S')
                else:
                    # Already formatted or other format
                    return value
            return str(value)
        except Exception as e:
            self.logger.error(f"Time formatting error: {e}")
            return str(value)


class ImageWindow(QLabel):
    def __init__(self, image_path, box=None, zone_coords=None, parent=None):
        super().__init__(parent)
        # Показывать как отдельное окно
        self.setWindowTitle('Image')
        try:
            self.setWindowFlag(Qt.Window, True)
            self.setAttribute(Qt.WidgetAttribute.WA_DeleteOnClose, True)
        except Exception:
            pass
        self.setFixedSize(900, 600)
        self.image_path = image_path
        self.zone_coords = zone_coords
        
        # Load image
        pixmap = QPixmap(image_path)
        if pixmap.isNull():
            try:
                self.logger.error(f"Image loading error: {image_path}")
            except Exception:
                pass
            # даже если не удалось загрузить, показываем окно с сообщением
            self.label = QLabel(f"Image not found:\n{image_path}")
            self.label.setAlignment(Qt.AlignmentFlag.AlignCenter)
            self.layout = QVBoxLayout()
            self.layout.addWidget(self.label)
            self.setLayout(self.layout)
            return
            
        # Compute target rect in window
        win_w, win_h = self.width(), self.height()
        img_w, img_h = pixmap.width(), pixmap.height()
        scale = min(win_w / img_w, win_h / img_h)
        draw_w = int(img_w * scale)
        draw_h = int(img_h * scale)
        draw_x = (win_w - draw_w) // 2
        draw_y = (win_h - draw_h) // 2

        # Create canvas pixmap sized to window
        canvas = QPixmap(win_w, win_h)
        canvas.fill(QColor(0, 0, 0))
        painter = QPainter()
        try:
            painter.begin(canvas)
            # Draw image
            painter.drawPixmap(draw_x, draw_y, draw_w, draw_h, pixmap)
            # Draw overlays in same mapping
            if box:
                pen = QPen(QColor(0, 255, 0), 2)
                painter.setPen(pen)
                # Поддержка dict {'x','y','width','height'}, [x,y,w,h] и [x1,y1,x2,y2]
                x1=y1=x2=y2=None
                if isinstance(box, dict):
                    bx = box.get('x', 0); by = box.get('y', 0); bw = box.get('width', 0); bh = box.get('height', 0)
                    if max(bx, by, bw, bh) <= 1.0:
                        x1 = bx; y1 = by; x2 = bx + bw; y2 = by + bh
                    else:
                        # абсолютные пиксели → нормализуем от размера исходного pixmap
                        x1 = bx / img_w; y1 = by / img_h; x2 = (bx + bw) / img_w; y2 = (by + bh) / img_h
                elif isinstance(box, (list, tuple)) and len(box) == 4:
                    a,b,c,d = box
                    if max(a,b,c,d) <= 1.0:
                        # это [x1,y1,x2,y2] в нормализованных координатах
                        x1,y1,x2,y2 = a,b,c,d
                    else:
                        # это [x,y,w,h] в пикселях
                        x1 = a / img_w; y1 = b / img_h; x2 = (a + c) / img_w; y2 = (b + d) / img_h
                if None not in (x1,y1,x2,y2):
                    x = draw_x + int(x1 * draw_w)
                    y = draw_y + int(y1 * draw_h)
                    w = int((x2 - x1) * draw_w)
                    h = int((y2 - y1) * draw_h)
                    painter.drawRect(x, y, w, h)
            if self.zone_coords:
                pen = QPen(QColor(255, 0, 0), 2)
                painter.setPen(pen)
                pts = []
                for px, py in self.zone_coords:
                    if max(px, py) <= 1.0:
                        pts.append((draw_x + int(px * draw_w), draw_y + int(py * draw_h)))
                    else:
                        pts.append((int(px), int(py)))
                for i in range(len(pts)):
                    x1, y1 = pts[i]
                    x2, y2 = pts[(i+1) % len(pts)]
                    painter.drawLine(x1, y1, x2, y2)
        finally:
            if painter.isActive():
                painter.end()
        
        # Create label and set pixmap
        self.label = QLabel()
        self.label.setPixmap(canvas)
        
        # Setup layout
        self.layout = QVBoxLayout()
        self.layout.addWidget(self.label)
        self.setLayout(self.layout)

    def mouseDoubleClickEvent(self, event):
        self.hide()
        event.accept()


class EventsJournalJson(QWidget):
    def __init__(self, base_dir: str, parent=None, logger_name: str | None = None, parent_logger: logging.Logger | None = None):
        super().__init__(parent)
        base_name = "evileye.events_journal_json"
        full_name = f"{base_name}.{logger_name}" if logger_name else base_name
        self.logger = parent_logger or logging.getLogger(full_name)
        self.setWindowTitle('Events journal (JSON)')
        self.resize(1600, 600)
        self.base_dir = base_dir
        self.ds = JsonLabelJournalDataSource(base_dir)
        self.page = 0
        self.page_size = 50
        self.filters: Dict = {}
        
        # Store last data hash for efficient updates
        self.last_data_hash = None
        self.is_visible = False
        self._is_closing = False  # Flag to prevent operations during closing
        
        # Real-time update timer
        self.update_timer = QTimer()
        self.update_timer.timeout.connect(self._check_for_updates)
        self.update_timer.start(500)  # Check every 0.5 seconds for better responsiveness

        self._build_ui()
        self._reload_dates()
        self._reload_table()

    def _build_ui(self):
        self.layout = QVBoxLayout()

        toolbar = QHBoxLayout()
        # Remove the directory selection button - use base_dir directly
        # self.btn_open_dir = QPushButton('Open images dir')
        # self.btn_open_dir.clicked.connect(self._choose_dir)
        # toolbar.addWidget(self.btn_open_dir)

        self.cmb_date = QComboBox()
        self.cmb_date.currentTextChanged.connect(self._on_date_changed)
        toolbar.addWidget(self.cmb_date)

        self.cmb_type = QComboBox()
        # Добавляем типы для атрибутных/зон/FOV событий, чтобы их можно было явно отфильтровать
        self.cmb_type.addItems(['All', 'found', 'lost', 'attr_found', 'attr_lost', 'zone_entered', 'zone_left', 'fov_found', 'fov_lost', 'cam'])
        self.cmb_type.currentTextChanged.connect(self._on_filter_changed)
        toolbar.addWidget(self.cmb_type)

        self.layout.addLayout(toolbar)

        # Use objects journal structure: Time, Event, Information, Source, Time lost, Preview, Lost preview
        self.table = QTableWidget(0, 7)
        self.table.setHorizontalHeaderLabels(['Time', 'Event', 'Information', 'Source', 'Time lost', 'Preview', 'Lost preview'])
        h = self.table.horizontalHeader()
        v = self.table.verticalHeader()
        h.setSectionResizeMode(0, QHeaderView.ResizeMode.ResizeToContents)  # Time
        h.setSectionResizeMode(1, QHeaderView.ResizeMode.ResizeToContents)  # Event
        h.setSectionResizeMode(2, QHeaderView.ResizeMode.ResizeToContents)  # Information
        h.setSectionResizeMode(3, QHeaderView.ResizeMode.ResizeToContents)  # Source
        h.setSectionResizeMode(4, QHeaderView.ResizeMode.Stretch)  # Time lost
        h.setSectionResizeMode(5, QHeaderView.ResizeMode.Fixed)  # Preview
        h.setSectionResizeMode(6, QHeaderView.ResizeMode.Fixed)  # Lost preview
        # Match aspect ratio with other journals: width=300, height=150
        try:
            h.resizeSection(5, 300)
            h.resizeSection(6, 300)
        except Exception:
            pass
        try:
            v.setDefaultSectionSize(150)
        except Exception:
            pass
        self.layout.addWidget(self.table)

        # Set up image delegate for image columns (Preview and Lost preview)
        self.image_delegate = ImageDelegate(self.table, self.base_dir, logger_name="image_delegate", parent_logger=self.logger)
        self.table.setItemDelegateForColumn(5, self.image_delegate)  # Preview
        self.table.setItemDelegateForColumn(6, self.image_delegate)  # Lost preview

        # Set up datetime delegate for time columns
        self.datetime_delegate = DateTimeDelegate(self.table)
        self.table.setItemDelegateForColumn(0, self.datetime_delegate)  # Time
        self.table.setItemDelegateForColumn(4, self.datetime_delegate)  # Time lost

        # Connect double click signal - use cellDoubleClicked for QTableWidget
        self.table.cellDoubleClicked.connect(self._display_image)
        
        # Store image window reference
        self.image_win = None

        self.setLayout(self.layout)
        
        # Enable automatic updates
        self.table.setUpdatesEnabled(True)
        
        # Connect show event to force update
        # Note: showEvent will be overridden in the class definition
        
        # Connect visibility change event (only if signal exists)
        try:
            self.visibilityChanged.connect(self._on_visibility_changed)
        except AttributeError:
            self.logger.warning("visibilityChanged signal unavailable, skipping visibility tracking")
        
        # Connect focus change event for better responsiveness
        try:
            self.windowActivated.connect(self._on_window_activated)
        except AttributeError:
            self.logger.warning("windowActivated signal unavailable, skipping activation tracking")

    def _choose_dir(self):
        d = QFileDialog.getExistingDirectory(self, 'Select images base directory', self.base_dir)
        if d:
            self.base_dir = d
            self.ds.set_base_dir(d)
            self._reload_dates()
            self._reload_table()

    def _on_date_changed(self, text: str):
        self.ds.set_date(text if text and text != 'All' else None)
        self._reload_table()

    def _on_filter_changed(self, text: str):
        self.filters['event_type'] = None if text == 'All' else text
        self._reload_table()

    def _reload_dates(self):
        try:
            dates = self.ds.list_available_dates()
            self.cmb_date.clear()
            self.cmb_date.addItem('All')
            for d in dates:
                self.cmb_date.addItem(d)
        except Exception as e:
            self.logger.error(f"Date loading error: {e}")
            self.cmb_date.clear()
            self.cmb_date.addItem('All')

    def _check_for_updates(self):
        """Check if data has changed and reload if necessary"""
        try:
            # Check if widget is closing or destroyed
            if self._is_closing:
                return
            # Check if widget still exists and is valid
            if not hasattr(self, 'table') or self.table is None:
                return
            if not hasattr(self, 'ds') or self.ds is None:
                return
            
            # Get current data hash
            filters = {k: v for k, v in self.filters.items() if v}
            current_data = self.ds.fetch(self.page, self.page_size, filters, [])
            
            # Create a hash based on data count and latest timestamp
            if current_data:
                latest_ts = max(ev.get('ts', '') for ev in current_data)
                data_count = len(current_data)
                current_hash = hash(f"{data_count}_{latest_ts}")
            else:
                current_hash = hash("empty")
            
            # Always reload for visible windows, or if data changed
            if current_hash != self.last_data_hash or self.is_visible:
                if current_hash != self.last_data_hash:
                    #self.logger.debug(f"🔄 Data changed! Hash: {self.last_data_hash} -> {current_hash}")
                    self.last_data_hash = current_hash
                #else:
                #    self.logger.debug(f"🔄 Forcing update for visible window. Hash: {current_hash}")
                
                self._reload_table()
                # Force widget repaint only if widget still exists
                if hasattr(self, 'table') and self.table is not None:
                    self.table.viewport().update()
                    self.table.repaint()
        except Exception as e:
            self.logger.error(f"Update check error: {e}")

    def _reload_table(self):
        try:
            # Check if widget is closing or destroyed
            if self._is_closing:
                return
            # Check if widget still exists and is valid
            if not hasattr(self, 'table') or self.table is None:
                return
            if not hasattr(self, 'ds') or self.ds is None:
                return
            
            filters = {k: v for k, v in self.filters.items() if v}
            # Use empty sort list to avoid sorting errors with None values
            rows = self.ds.fetch(self.page, self.page_size, filters, [])
            # Показываем события детекторов и системные: attr_*, zone_*, fov_*, cam, sys
            rows = [ev for ev in rows if (
                (et := ev.get('event_type', '')) and (
                    et.startswith('attr') or et.startswith('zone') or et.startswith('fov') or et == 'cam' or et == 'sys'
                )
            )]
            # Короткая сводка по типам для диагностики (в логи)
            try:
                counts = {}
                for ev in rows:
                    et = ev.get('event_type', 'na')
                    counts[et] = counts.get(et, 0) + 1
                self.logger.info(f"JSON events summary: {counts}")
            except Exception:
                pass
            
            # Group paired events to show both Preview and Lost preview in one row
            grouped = {}
            cam_events = []
            sys_events = []
            for ev in rows:
                et = ev.get('event_type','')
                if et.startswith('attr'):
                    key = ('attr', ev.get('object_id'))
                elif et.startswith('zone'):
                    key = ('zone', ev.get('source_id'), ev.get('zone_id'))
                elif et.startswith('fov'):
                    key = ('fov', ev.get('source_id'), ev.get('object_id'))
                elif et == 'cam':
                    cam_events.append(ev)
                    continue
                elif et == 'sys':
                    sys_events.append(ev)
                    continue
                else:
                    continue
                bucket = grouped.setdefault(key, {'found': None, 'lost': None})
                if et in ('attr_found','zone_entered','fov_found'):
                    bucket['found'] = ev
                elif et in ('attr_lost','zone_left','fov_lost'):
                    bucket['lost'] = ev

            table_rows = []
            for key, pair in grouped.items():
                kind = key[0]
                found_ev = pair['found']
                lost_ev = pair['lost']
                base = found_ev or lost_ev
                if not base:
                    continue
                if kind == 'attr':
                    event_name = 'AttributeEvent'
                    info = f"AttributeEvent name={base.get('event_name','')}; obj={base.get('object_id')}; class={base.get('class_name', base.get('class_id',''))}; attrs={base.get('attrs', [])}"
                elif kind == 'zone':
                    event_name = 'ZoneEvent'
                    info = f"ZoneEvent obj={base.get('object_id')} zone={base.get('zone_id','')}"
                else:
                    event_name = 'FOVEvent'
                    info = f"FOVEvent obj={base.get('object_id')}"

                row_data = {
                    'source': base.get('source_name') or str(base.get('source_id', '')),
                    'event': event_name,
                    'information': info,
                    'time': (found_ev.get('ts') if found_ev else base.get('ts','')),
                    'time_lost': (lost_ev.get('ts') if lost_ev else ''),
                    'preview': (found_ev.get('image_filename','') if found_ev else ''),
                    'lost_preview': (lost_ev.get('image_filename','') if lost_ev else ''),
                    'found_event': found_ev,
                    'lost_event': lost_ev
                }
                table_rows.append(row_data)

            # Add camera events as standalone rows
            for ev in cam_events:
                table_rows.append({
                    'source': ev.get('camera_full_address', ''),
                    'event': 'CameraEvent',
                    'information': f"Camera {ev.get('camera_full_address')} status={ev.get('connection_status')}",
                    'time': ev.get('ts',''),
                    'time_lost': '',
                    'preview': '',
                    'lost_preview': '',
                    'found_event': None,
                    'lost_event': None
                })

            # Добавляем системные события отдельными строками
            for ev in sys_events:
                table_rows.append({
                    'source': 'System',
                    'event': 'SystemEvent',
                    'information': f"System {ev.get('system_event','')}",
                    'time': ev.get('ts',''),
                    'time_lost': '',
                    'preview': '',
                    'lost_preview': '',
                    'found_event': None,
                    'lost_event': None
                })

            # Sort all rows by time desc to ensure recent System/Zone/etc are visible on first page
            try:
                table_rows.sort(key=lambda r: (r.get('time') or ''), reverse=True)
            except Exception:
                pass
            
            self.table.setRowCount(len(table_rows))
            for r, row_data in enumerate(table_rows):
                # Time column (0)
                self.table.setItem(r, 0, QTableWidgetItem(str(row_data['time'])))
                
                # Event column (1)
                self.table.setItem(r, 1, QTableWidgetItem(row_data['event']))
                
                # Information column (2)
                self.table.setItem(r, 2, QTableWidgetItem(row_data['information']))
                
                # Source column (3)
                self.table.setItem(r, 3, QTableWidgetItem(str(row_data.get('source', ''))))
                
                # Time lost column (4)
                self.table.setItem(r, 4, QTableWidgetItem(str(row_data['time_lost'])))
                
                # Preview column (found image)
                if row_data['preview']:
                    found_event = row_data.get('found_event')
                    date_folder = found_event.get('date_folder', '') if found_event else ''
                    prev = row_data['preview']
                    if os.path.isabs(prev):
                        img_path = prev
                    elif prev.startswith('images' + os.sep) or prev.startswith('images/'):
                        img_path = os.path.join(self.base_dir, prev)
                    else:
                        img_path = os.path.join(self.base_dir, 'images', date_folder, prev)
                    item = QTableWidgetItem(img_path)
                    # Store event data for double click functionality
                    item.setData(Qt.ItemDataRole.UserRole, found_event)
                    self.table.setItem(r, 5, item)
                else:
                    # Store empty string but still create item for delegate
                    item = QTableWidgetItem('')
                    found_event = row_data.get('found_event')
                    if found_event:
                        item.setData(Qt.ItemDataRole.UserRole, found_event)
                    self.table.setItem(r, 5, item)
                
                # Lost preview column
                if row_data['lost_preview']:
                    lost_event = row_data.get('lost_event')
                    date_folder = lost_event.get('date_folder', '') if lost_event else ''
                    prev = row_data['lost_preview']
                    if os.path.isabs(prev):
                        img_path = prev
                    elif prev.startswith('images' + os.sep) or prev.startswith('images/'):
                        img_path = os.path.join(self.base_dir, prev)
                    else:
                        img_path = os.path.join(self.base_dir, 'images', date_folder, prev)
                    item = QTableWidgetItem(img_path)
                    # Store event data for double click functionality
                    item.setData(Qt.ItemDataRole.UserRole, lost_event)
                    self.table.setItem(r, 6, item)
                else:
                    # Store empty string but still create item for delegate
                    item = QTableWidgetItem('')
                    lost_event = row_data.get('lost_event')
                    if lost_event:
                        item.setData(Qt.ItemDataRole.UserRole, lost_event)
                    self.table.setItem(r, 6, item)
                
                # Set row height for image display
                self.table.setRowHeight(r, 150)
            
            # Force widget update to ensure changes are visible
            # Only update if widget still exists and is not closing
            if not self._is_closing and hasattr(self, 'table') and self.table is not None:
                try:
                    self.table.viewport().update()
                    self.table.update()
                    
                    # Force repaint
                    self.table.repaint()
                    # Don't call processEvents() here as it can cause segfault
                    # Events will be processed naturally by the event loop
                except (RuntimeError, AttributeError):
                    # Widget may have been destroyed
                    pass
            
        except Exception as e:
            self.logger.error(f"Table data loading error: {e}")
    
    def _on_visibility_changed(self, visible):
        """Handle visibility change to force update when window becomes visible"""
        self.is_visible = visible
        if visible:
            self.logger.info("Window became visible, forced update...")
            self._reload_table()
    
    def force_update(self):
        """Force immediate update of the journal"""
        self.logger.info("Forced update requested...")
        self._reload_table()
        self.table.viewport().update()
        self.table.repaint()
    
    def _on_window_activated(self):
        """Handle window activation to force update"""
        self.logger.info("Window activated, forced update...")
        self._reload_table()
        self.table.viewport().update()
        self.table.repaint()

    def showEvent(self, event):
        """Start update timer when window is shown"""
        super().showEvent(event)
        if hasattr(self, 'update_timer'):
            self.update_timer.stop()  # Stop first to ensure clean restart
            self.update_timer.start(1000)  # Restart timer every 1 second
        # Force immediate reload to show latest data
        self._reload_table()

    def hideEvent(self, event):
        """Stop update timer when window is hidden"""
        super().hideEvent(event)
        if hasattr(self, 'update_timer'):
            self.update_timer.stop()

    def closeEvent(self, event):
        # Устанавливаем флаг закрытия перед остановкой таймера
        self._is_closing = True
        
        # Останавливаем таймер перед закрытием
        if hasattr(self, 'update_timer'):
            self.update_timer.stop()
            # Не вызываем processEvents() здесь, так как это может вызвать segfault
        # Закрываем data source
        if hasattr(self, 'ds') and self.ds:
            try:
                self.ds.close()
            except Exception:
                pass
        super().closeEvent(event)

    @pyqtSlot(int, int)
    def _display_image(self, row, col):
        """Display full image on double click (similar to database journal)"""
        if col != 5 and col != 6:  # Only Preview and Lost preview columns
            return

        # Get path from table item
        path = None
        table_item = self.table.item(row, col)
        if table_item:
            path = table_item.text()
        if not path:
            return

        # Get row data to find bounding box
        if row >= self.table.rowCount():
            return

        # Get event data from the row
        found_event = None
        lost_event = None
        
        # Try to get event data from table items (stored in UserRole)
        found_item = self.table.item(row, 5)  # Preview column
        lost_item = self.table.item(row, 6)    # Lost preview column
        
        if found_item:
            found_event = found_item.data(Qt.ItemDataRole.UserRole)
        if lost_item:
            lost_event = lost_item.data(Qt.ItemDataRole.UserRole)

        # Get bounding box from event data
        box = None
        if col == 5 and found_event:  # Preview column
            bbox_data = found_event.get('bounding_box')
            if bbox_data:
                if isinstance(bbox_data, dict):
                    # Convert dict format to list format
                    x = bbox_data.get('x', 0)
                    y = bbox_data.get('y', 0)
                    w = bbox_data.get('width', 0)
                    h = bbox_data.get('height', 0)
                    box = [x, y, w, h]
                elif isinstance(bbox_data, list) and len(bbox_data) == 4:
                    box = bbox_data
        elif col == 6 and lost_event:  # Lost preview column
            bbox_data = lost_event.get('bounding_box')
            if bbox_data:
                if isinstance(bbox_data, dict):
                    # Convert dict format to list format
                    x = bbox_data.get('x', 0)
                    y = bbox_data.get('y', 0)
                    w = bbox_data.get('width', 0)
                    h = bbox_data.get('height', 0)
                    box = [x, y, w, h]
                elif isinstance(bbox_data, list) and len(bbox_data) == 4:
                    box = bbox_data

        # Convert preview path to frame path (similar to database journal)
        image_path = path
        if 'preview' in path:
            # Extract filename and convert preview to frame
            dir_path, filename = os.path.split(path)
            if 'preview' in filename:
                # Replace 'preview' with 'frame' in filename
                new_filename = filename.replace('preview', 'frame')
                
                # Convert directory path: new structure FoundPreviews->FoundFrames, LostPreviews->LostFrames
                if 'FoundPreviews' in dir_path:
                    new_dir_path = dir_path.replace('FoundPreviews', 'FoundFrames')
                    image_path = os.path.join(new_dir_path, new_filename)
                elif 'LostPreviews' in dir_path:
                    new_dir_path = dir_path.replace('LostPreviews', 'LostFrames')
                    image_path = os.path.join(new_dir_path, new_filename)
                elif 'previews' in dir_path:
                    # Legacy support
                    new_dir_path = dir_path.replace('previews', 'frames')
                    image_path = os.path.join(new_dir_path, new_filename)
                else:
                    # If no 'previews' in path, just replace filename
                    image_path = os.path.join(dir_path, new_filename)

        # Check if frame image exists, otherwise use preview
        if not os.path.exists(image_path):
            self.logger.warning(f"Frame image not found: {image_path}, using preview: {path}")
            image_path = path

        # Zone coords if present
        zone_coords = None
        if col == 5 and found_event and found_event.get('zone_coords'):
            zone_coords = found_event.get('zone_coords')
        elif col == 6 and lost_event and lost_event.get('zone_coords'):
            zone_coords = lost_event.get('zone_coords')

        # If box is in absolute pixels (>1), normalize using actual image size for correct scaling
        if box and max(box) > 1:
            try:
                from PyQt6.QtGui import QPixmap as _QPixmap
            except ImportError:
                from PyQt5.QtGui import QPixmap as _QPixmap
            pix = _QPixmap(image_path)
            if not pix.isNull() and pix.width() > 0 and pix.height() > 0:
                bx, by, bw, bh = box
                box = [bx / pix.width(), by / pix.height(), bw / pix.width(), bh / pix.height()]

        # Create and show image window
        self.image_win = ImageWindow(image_path, box, zone_coords)
        self.image_win.show()





"""
Dialogs module for EvilEye GUI

Модуль содержит диалоговые окна для GUI приложения.
"""

from .save_confirmation_dialog import SaveConfirmationDialog, SaveAsDialog
from .config_restore_dialog import ConfigRestoreDialog
from .config_compare_dialog import ConfigCompareDialog
from .job_details_dialog import JobDetailsDialog
from .export_history_dialog import ExportHistoryDialog
from .class_mapping_dialog import ClassMappingDialog

__all__ = [
    'SaveConfirmationDialog',
    'SaveAsDialog',
    'ConfigRestoreDialog',
    'ConfigCompareDialog',
    'JobDetailsDialog',
    'ExportHistoryDialog',
    'ClassMappingDialog'
]



from typing import List
import onnxruntime as ort
import numpy as np
import cv2
import albumentations as A
from albumentations.pytorch import ToTensorV2
from .track_encoder import TrackEncoder
import os
import requests
from tqdm import tqdm
from ...core.logger import get_module_logger

url = "https://github.com/aicommunity/EvilEye/releases/download/dev/osnet_ain_x1_0_M.onnx"

class OnnxEncoder(TrackEncoder):
    def __init__(self, model_path: str, batch_size: int = 1):
        self.logger = get_module_logger("onnx_encoder")
        self.batch_size = batch_size

        # Resolve relative onnx path to current working directory for access
        model_path_resolved = model_path
        if not os.path.isabs(model_path_resolved):
            model_path_resolved = os.path.join(os.getcwd(), model_path_resolved)

        if not os.path.exists(model_path_resolved):
            # Create directory if it doesn't exist and path is not just filename
            dirname = os.path.dirname(model_path_resolved)
            if dirname:
                os.makedirs(dirname, exist_ok=True)
            self.logger.info(f"File not found. Downloading to {model_path_resolved}...")
            response = requests.get(url, stream=True)
            response.raise_for_status()
            total_size = int(response.headers.get('content-length', 0))
            block_size = 8192

            with open(model_path_resolved, "wb") as f, tqdm(
                    total=total_size, unit='B', unit_scale=True, desc="Загрузка", ncols=80
            ) as pbar:
                for chunk in response.iter_content(chunk_size=block_size):
                    if chunk:
                        f.write(chunk)
                        pbar.update(len(chunk))
            self.logger.info("Download successful")

        self.session = ort.InferenceSession(model_path)
        self.input_name = self.session.get_inputs()[0].name
        self.output_name = self.session.get_outputs()[0].name
        self.image_augmentation = A.Compose(
            [A.Resize(256, 128), A.Normalize(), ToTensorV2()]
        )

    def inference(self, img: np.ndarray, dets: np.ndarray) -> np.ndarray:
        """inference encoder and get features for each object

        :param img: a current frame
        :param dets: detections (Nx4) that have format [x_center. y_center, w, h]
        :return: features (NxF)
        """
        features = []
        
        for i in range(0, len(dets), self.batch_size):
            batch_dets = dets[i: i + self.batch_size]
            batch_crops = self._dets2crops(img, batch_dets)
            batch = self._crop2batch(batch_crops)
            output_array = self.session.run(
                [self.output_name], 
                {self.input_name: batch}
            )
            features += [f for f in output_array[0]]

        features = np.array(features)
        return features
    
    def _dets2crops(self, img, dets) -> List[np.ndarray]:
        crops = []

        for det in dets:
            xc, yc, w, h = det[:4]
            x = xc - w / 2
            y = yc - h / 2
            x, y, w, h = map(int, [x, y, w, h])

            crop = img[y: y + h, x: x + w]
            crops.append(crop)
        
        return crops
    
    def _crop2batch(self, crops: List[np.ndarray]) -> np.ndarray:
        preprocessed_crops = [self._preprocess(c) for c in crops]
        batch = np.concatenate(preprocessed_crops, axis=0)
        return batch

    def _preprocess(self, image: np.ndarray) -> np.ndarray:
        # Assuming the model expects a 224x224 RGB image
        image = cv2.cvtColor(image, cv2.COLOR_BGR2RGB)
        image = self.image_augmentation(image=np.array(image))["image"]
        image = np.expand_dims(image, axis=0)
        return image
    



import json
import os
from typing import Dict, List, Optional, Tuple
try:
    from PyQt6.QtWidgets import (
        QDialog, QVBoxLayout, QHBoxLayout, QPushButton, QLabel, 
        QTableWidget, QTableWidgetItem, QHeaderView, QGroupBox,
        QFormLayout, QDialogButtonBox, QMessageBox, QFileDialog,
        QLineEdit, QSpinBox, QComboBox, QCheckBox, QTextEdit,
        QTabWidget, QWidget, QSplitter, QListWidget, QListWidgetItem
    )
    from PyQt6.QtCore import Qt, pyqtSignal, QTimer
    from PyQt6.QtGui import QFont, QColor
    pyqt_version = 6
except ImportError:
    from PyQt5.QtWidgets import (
        QDialog, QVBoxLayout, QHBoxLayout, QPushButton, QLabel, 
        QTableWidget, QTableWidgetItem, QHeaderView, QGroupBox,
        QFormLayout, QDialogButtonBox, QMessageBox, QFileDialog,
        QLineEdit, QSpinBox, QComboBox, QCheckBox, QTextEdit,
        QTabWidget, QWidget, QSplitter, QListWidget, QListWidgetItem
    )
    from PyQt5.QtCore import Qt, pyqtSignal, QTimer
    from PyQt5.QtGui import QFont, QColor
    pyqt_version = 5


class ClassMappingDialog(QDialog):
    """Диалог для редактирования маппинга классов"""
    
    class_mapping_updated = pyqtSignal(dict)  # Сигнал с обновленным маппингом
    
    def __init__(self, parent=None, initial_mapping: Optional[Dict] = None):
        super().__init__(parent)
        self.setWindowTitle("Class Mapping Editor")
        self.setModal(True)
        self.resize(800, 600)
        
        self.class_mapping = initial_mapping or {}
        self._setup_ui()
        self._load_mapping()
        
    def _setup_ui(self):
        """Настройка пользовательского интерфейса"""
        layout = QVBoxLayout(self)
        
        # Верхняя панель с кнопками
        top_layout = QHBoxLayout()
        
        self.add_class_btn = QPushButton("Добавить класс")
        self.add_class_btn.clicked.connect(self._add_class)
        top_layout.addWidget(self.add_class_btn)
        
        self.remove_class_btn = QPushButton("Удалить класс")
        self.remove_class_btn.clicked.connect(self._remove_class)
        top_layout.addWidget(self.remove_class_btn)
        
        self.import_from_model_btn = QPushButton("Импорт из модели")
        self.import_from_model_btn.clicked.connect(self._import_from_model)
        top_layout.addWidget(self.import_from_model_btn)
        
        self.import_from_file_btn = QPushButton("Импорт из файла")
        self.import_from_file_btn.clicked.connect(self._import_from_file)
        top_layout.addWidget(self.import_from_file_btn)
        
        self.export_btn = QPushButton("Экспорт")
        self.export_btn.clicked.connect(self._export_mapping)
        top_layout.addWidget(self.export_btn)
        
        self.clear_btn = QPushButton("Очистить")
        self.clear_btn.clicked.connect(self._clear_mapping)
        top_layout.addWidget(self.clear_btn)
        
        top_layout.addStretch()
        layout.addLayout(top_layout)
        
        # Основная область
        main_layout = QHBoxLayout()
        
        # Левая панель - таблица классов
        left_panel = QVBoxLayout()
        
        # Таблица классов
        self.classes_table = QTableWidget()
        self.classes_table.setColumnCount(3)
        self.classes_table.setHorizontalHeaderLabels(["Class ID", "Class Name", "Description"])
        self.classes_table.horizontalHeader().setSectionResizeMode(QHeaderView.ResizeMode.Stretch)
        self.classes_table.setSelectionBehavior(QTableWidget.SelectionBehavior.SelectRows)
        self.classes_table.setAlternatingRowColors(True)
        self.classes_table.itemChanged.connect(self._on_table_item_changed)
        self.classes_table.itemSelectionChanged.connect(self._on_selection_changed)
        
        left_panel.addWidget(QLabel("Class Mapping:"))
        left_panel.addWidget(self.classes_table)
        
        # Кнопки управления таблицей
        table_buttons_layout = QHBoxLayout()
        
        self.move_up_btn = QPushButton("↑")
        self.move_up_btn.clicked.connect(self._move_up)
        self.move_up_btn.setEnabled(False)
        table_buttons_layout.addWidget(self.move_up_btn)
        
        self.move_down_btn = QPushButton("↓")
        self.move_down_btn.clicked.connect(self._move_down)
        self.move_down_btn.setEnabled(False)
        table_buttons_layout.addWidget(self.move_down_btn)
        
        table_buttons_layout.addStretch()
        left_panel.addLayout(table_buttons_layout)
        
        main_layout.addLayout(left_panel, 2)
        
        # Правая панель - настройки и предпросмотр
        right_panel = QVBoxLayout()
        
        # Группа настроек
        settings_group = QGroupBox("Настройки")
        settings_layout = QFormLayout(settings_group)
        
        self.auto_id_check = QCheckBox("Автоматическая нумерация ID")
        self.auto_id_check.setChecked(True)
        self.auto_id_check.toggled.connect(self._on_auto_id_toggled)
        settings_layout.addRow("", self.auto_id_check)
        
        self.start_id_spin = QSpinBox()
        self.start_id_spin.setRange(0, 10000)
        self.start_id_spin.setValue(0)
        self.start_id_spin.setEnabled(False)
        settings_layout.addRow("Начальный ID:", self.start_id_spin)
        
        self.validate_mapping_btn = QPushButton("Валидация")
        self.validate_mapping_btn.clicked.connect(self._validate_mapping)
        settings_layout.addRow("", self.validate_mapping_btn)
        
        right_panel.addWidget(settings_group)
        
        # Группа предпросмотра
        preview_group = QGroupBox("Предпросмотр")
        preview_layout = QVBoxLayout(preview_group)
        
        self.preview_text = QTextEdit()
        self.preview_text.setReadOnly(True)
        self.preview_text.setMaximumHeight(200)
        preview_layout.addWidget(self.preview_text)
        
        self.update_preview_btn = QPushButton("Обновить предпросмотр")
        self.update_preview_btn.clicked.connect(self._update_preview)
        preview_layout.addWidget(self.update_preview_btn)
        
        right_panel.addWidget(preview_group)
        
        # Группа статистики
        stats_group = QGroupBox("Статистика")
        stats_layout = QFormLayout(stats_group)
        
        self.total_classes_label = QLabel("0")
        stats_layout.addRow("Всего классов:", self.total_classes_label)
        
        self.max_id_label = QLabel("0")
        stats_layout.addRow("Максимальный ID:", self.max_id_label)
        
        self.duplicate_ids_label = QLabel("0")
        self.duplicate_ids_label.setStyleSheet("color: red;")
        stats_layout.addRow("Дублирующиеся ID:", self.duplicate_ids_label)
        
        right_panel.addWidget(stats_group)
        
        right_panel.addStretch()
        main_layout.addLayout(right_panel, 1)
        
        layout.addLayout(main_layout)
        
        # Кнопки диалога
        button_box = QDialogButtonBox(
            QDialogButtonBox.StandardButton.Ok | QDialogButtonBox.StandardButton.Cancel
        )
        button_box.accepted.connect(self._accept_dialog)
        button_box.rejected.connect(self.reject)
        layout.addWidget(button_box)
    
    def _load_mapping(self):
        """Загрузить маппинг в таблицу"""
        self.classes_table.setRowCount(len(self.class_mapping))
        
        for row, (class_id, class_name) in enumerate(self.class_mapping.items()):
            # Class ID
            id_item = QTableWidgetItem(str(class_id))
            id_item.setFlags(id_item.flags() & ~Qt.ItemFlag.ItemIsEditable if self.auto_id_check.isChecked() else id_item.flags())
            self.classes_table.setItem(row, 0, id_item)
            
            # Class Name
            name_item = QTableWidgetItem(str(class_name))
            self.classes_table.setItem(row, 1, name_item)
            
            # Description (пустое по умолчанию)
            desc_item = QTableWidgetItem("")
            self.classes_table.setItem(row, 2, desc_item)
        
        self._update_statistics()
        self._update_preview()
    
    def _add_class(self):
        """Добавить новый класс"""
        row = self.classes_table.rowCount()
        self.classes_table.insertRow(row)
        
        # Определяем следующий ID
        if self.auto_id_check.isChecked():
            next_id = self._get_next_id()
        else:
            next_id = self.start_id_spin.value()
        
        # Class ID
        id_item = QTableWidgetItem(str(next_id))
        if self.auto_id_check.isChecked():
            id_item.setFlags(id_item.flags() & ~Qt.ItemFlag.ItemIsEditable)
        self.classes_table.setItem(row, 0, id_item)
        
        # Class Name
        name_item = QTableWidgetItem(f"class_{next_id}")
        self.classes_table.setItem(row, 1, name_item)
        
        # Description
        desc_item = QTableWidgetItem("")
        self.classes_table.setItem(row, 2, desc_item)
        
        self._update_statistics()
        self._update_preview()
    
    def _remove_class(self):
        """Удалить выбранный класс"""
        current_row = self.classes_table.currentRow()
        if current_row >= 0:
            class_name = self.classes_table.item(current_row, 1).text()
            
            reply = QMessageBox.question(
                self,
                "Подтверждение",
                f"Удалить класс '{class_name}'?",
                QMessageBox.StandardButton.Yes | QMessageBox.StandardButton.No
            )
            
            if reply == QMessageBox.StandardButton.Yes:
                self.classes_table.removeRow(current_row)
                self._update_statistics()
                self._update_preview()
    
    def _move_up(self):
        """Переместить выбранную строку вверх"""
        current_row = self.classes_table.currentRow()
        if current_row > 0:
            self._swap_rows(current_row, current_row - 1)
            self.classes_table.selectRow(current_row - 1)
    
    def _move_down(self):
        """Переместить выбранную строку вниз"""
        current_row = self.classes_table.currentRow()
        if current_row < self.classes_table.rowCount() - 1:
            self._swap_rows(current_row, current_row + 1)
            self.classes_table.selectRow(current_row + 1)
    
    def _swap_rows(self, row1: int, row2: int):
        """Поменять местами две строки"""
        for col in range(self.classes_table.columnCount()):
            item1 = self.classes_table.takeItem(row1, col)
            item2 = self.classes_table.takeItem(row2, col)
            self.classes_table.setItem(row1, col, item2)
            self.classes_table.setItem(row2, col, item1)
    
    def _on_table_item_changed(self, item):
        """Обработка изменения элемента таблицы"""
        if item.column() == 0:  # Class ID
            # Проверяем уникальность ID
            self._validate_ids()
        
        self._update_statistics()
        self._update_preview()
    
    def _on_selection_changed(self):
        """Обработка изменения выбора"""
        current_row = self.classes_table.currentRow()
        has_selection = current_row >= 0
        
        self.remove_class_btn.setEnabled(has_selection)
        self.move_up_btn.setEnabled(has_selection and current_row > 0)
        self.move_down_btn.setEnabled(has_selection and current_row < self.classes_table.rowCount() - 1)
    
    def _on_auto_id_toggled(self, checked):
        """Обработка переключения автоматической нумерации"""
        self.start_id_spin.setEnabled(not checked)
        
        # Обновляем флаги редактирования для всех ID
        for row in range(self.classes_table.rowCount()):
            id_item = self.classes_table.item(row, 0)
            if id_item:
                if checked:
                    id_item.setFlags(id_item.flags() & ~Qt.ItemFlag.ItemIsEditable)
                else:
                    id_item.setFlags(id_item.flags() | Qt.ItemFlag.ItemIsEditable)
    
    def _get_next_id(self) -> int:
        """Получить следующий доступный ID"""
        used_ids = set()
        for row in range(self.classes_table.rowCount()):
            id_item = self.classes_table.item(row, 0)
            if id_item:
                try:
                    used_ids.add(int(id_item.text()))
                except ValueError:
                    pass
        
        next_id = self.start_id_spin.value()
        while next_id in used_ids:
            next_id += 1
        
        return next_id
    
    def _validate_ids(self):
        """Проверить уникальность ID"""
        ids = []
        for row in range(self.classes_table.rowCount()):
            id_item = self.classes_table.item(row, 0)
            if id_item:
                try:
                    class_id = int(id_item.text())
                    if class_id in ids:
                        # Подсвечиваем дублирующиеся ID
                        id_item.setBackground(QColor(255, 200, 200))
                    else:
                        id_item.setBackground(QColor(255, 255, 255))
                        ids.append(class_id)
                except ValueError:
                    id_item.setBackground(QColor(255, 200, 200))
    
    def _validate_mapping(self):
        """Валидация маппинга"""
        errors = []
        warnings = []
        
        # Проверяем уникальность ID
        ids = []
        for row in range(self.classes_table.rowCount()):
            id_item = self.classes_table.item(row, 0)
            name_item = self.classes_table.item(row, 1)
            
            if not id_item or not name_item:
                continue
            
            try:
                class_id = int(id_item.text())
                class_name = name_item.text().strip()
                
                if not class_name:
                    errors.append(f"Строка {row + 1}: пустое название класса")
                
                if class_id in ids:
                    errors.append(f"Строка {row + 1}: дублирующийся ID {class_id}")
                else:
                    ids.append(class_id)
                
                if class_id < 0:
                    warnings.append(f"Строка {row + 1}: отрицательный ID {class_id}")
                
            except ValueError:
                errors.append(f"Строка {row + 1}: неверный формат ID")
        
        # Проверяем уникальность названий
        names = []
        for row in range(self.classes_table.rowCount()):
            name_item = self.classes_table.item(row, 1)
            if name_item:
                class_name = name_item.text().strip()
                if class_name and class_name in names:
                    warnings.append(f"Строка {row + 1}: дублирующееся название '{class_name}'")
                elif class_name:
                    names.append(class_name)
        
        # Показываем результаты валидации
        if errors or warnings:
            message = "Результаты валидации:\n\n"
            if errors:
                message += "Ошибки:\n" + "\n".join(f"• {error}" for error in errors) + "\n\n"
            if warnings:
                message += "Предупреждения:\n" + "\n".join(f"• {warning}" for warning in warnings)
            
            QMessageBox.warning(self, "Валидация", message)
        else:
            QMessageBox.information(self, "Валидация", "Маппинг классов корректен!")
    
    def _update_statistics(self):
        """Обновить статистику"""
        total_classes = self.classes_table.rowCount()
        self.total_classes_label.setText(str(total_classes))
        
        max_id = 0
        duplicate_count = 0
        ids = []
        
        for row in range(total_classes):
            id_item = self.classes_table.item(row, 0)
            if id_item:
                try:
                    class_id = int(id_item.text())
                    max_id = max(max_id, class_id)
                    if class_id in ids:
                        duplicate_count += 1
                    else:
                        ids.append(class_id)
                except ValueError:
                    pass
        
        self.max_id_label.setText(str(max_id))
        self.duplicate_ids_label.setText(str(duplicate_count))
    
    def _update_preview(self):
        """Обновить предпросмотр"""
        mapping = {}
        for row in range(self.classes_table.rowCount()):
            id_item = self.classes_table.item(row, 0)
            name_item = self.classes_table.item(row, 1)
            
            if id_item and name_item:
                try:
                    class_id = int(id_item.text())
                    class_name = name_item.text().strip()
                    if class_name:
                        mapping[class_id] = class_name
                except ValueError:
                    pass
        
        # Форматируем предпросмотр
        preview_text = "Class Mapping:\n\n"
        for class_id, class_name in sorted(mapping.items()):
            preview_text += f"{class_id}: {class_name}\n"
        
        # JSON предпросмотр
        preview_text += "\nJSON Format:\n"
        preview_text += json.dumps(mapping, indent=2, ensure_ascii=False)
        
        self.preview_text.setPlainText(preview_text)
    
    def _import_from_model(self):
        """Импорт классов из модели"""
        file_path, _ = QFileDialog.getOpenFileName(
            self,
            "Выберите файл модели",
            "",
            "Model Files (*.pt *.pth *.onnx);;All Files (*)"
        )
        
        if file_path:
            try:
                # Здесь должна быть логика извлечения классов из модели
                # Пока что показываем заглушку
                QMessageBox.information(
                    self,
                    "Импорт из модели",
                    f"Функция импорта из модели {file_path} будет реализована в будущих версиях.\n"
                    f"Пока что используйте импорт из файла."
                )
            except Exception as e:
                QMessageBox.critical(self, "Ошибка", f"Ошибка импорта из модели: {str(e)}")
    
    def _import_from_file(self):
        """Импорт маппинга из файла"""
        file_path, _ = QFileDialog.getOpenFileName(
            self,
            "Выберите файл маппинга",
            "",
            "JSON Files (*.json);;Text Files (*.txt);;All Files (*)"
        )
        
        if file_path:
            try:
                with open(file_path, 'r', encoding='utf-8') as f:
                    if file_path.endswith('.json'):
                        mapping = json.load(f)
                    else:
                        # Пытаемся загрузить как JSON
                        mapping = json.load(f)
                
                # Очищаем текущую таблицу
                self.classes_table.setRowCount(0)
                
                # Добавляем импортированные классы
                for class_id, class_name in mapping.items():
                    self._add_class()
                    row = self.classes_table.rowCount() - 1
                    
                    id_item = QTableWidgetItem(str(class_id))
                    if self.auto_id_check.isChecked():
                        id_item.setFlags(id_item.flags() & ~Qt.ItemFlag.ItemIsEditable)
                    self.classes_table.setItem(row, 0, id_item)
                    
                    name_item = QTableWidgetItem(str(class_name))
                    self.classes_table.setItem(row, 1, name_item)
                
                self._update_statistics()
                self._update_preview()
                
                QMessageBox.information(self, "Успех", f"Маппинг импортирован из {file_path}")
                
            except Exception as e:
                QMessageBox.critical(self, "Ошибка", f"Ошибка импорта: {str(e)}")
    
    def _export_mapping(self):
        """Экспорт маппинга в файл"""
        mapping = self._get_current_mapping()
        if not mapping:
            QMessageBox.information(self, "Информация", "Нет данных для экспорта")
            return
        
        file_path, _ = QFileDialog.getSaveFileName(
            self,
            "Экспорт маппинга",
            "class_mapping.json",
            "JSON Files (*.json);;Text Files (*.txt);;All Files (*)"
        )
        
        if file_path:
            try:
                with open(file_path, 'w', encoding='utf-8') as f:
                    if file_path.endswith('.json'):
                        json.dump(mapping, f, indent=2, ensure_ascii=False)
                    else:
                        # Экспорт в текстовом формате
                        for class_id, class_name in sorted(mapping.items()):
                            f.write(f"{class_id}: {class_name}\n")
                
                QMessageBox.information(self, "Успех", f"Маппинг экспортирован в {file_path}")
                
            except Exception as e:
                QMessageBox.critical(self, "Ошибка", f"Ошибка экспорта: {str(e)}")
    
    def _clear_mapping(self):
        """Очистить маппинг"""
        reply = QMessageBox.question(
            self,
            "Подтверждение",
            "Вы уверены, что хотите очистить весь маппинг?",
            QMessageBox.StandardButton.Yes | QMessageBox.StandardButton.No
        )
        
        if reply == QMessageBox.StandardButton.Yes:
            self.classes_table.setRowCount(0)
            self._update_statistics()
            self._update_preview()
    
    def _get_current_mapping(self) -> Dict[int, str]:
        """Получить текущий маппинг из таблицы"""
        mapping = {}
        for row in range(self.classes_table.rowCount()):
            id_item = self.classes_table.item(row, 0)
            name_item = self.classes_table.item(row, 1)
            
            if id_item and name_item:
                try:
                    class_id = int(id_item.text())
                    class_name = name_item.text().strip()
                    if class_name:
                        mapping[class_id] = class_name
                except ValueError:
                    pass
        
        return mapping
    
    def _accept_dialog(self):
        """Принять диалог и отправить сигнал"""
        mapping = self._get_current_mapping()
        self.class_mapping_updated.emit(mapping)
        self.accept()
    
    def get_mapping(self) -> Dict[int, str]:
        """Получить текущий маппинг"""
        return self._get_current_mapping()
    
    def set_mapping(self, mapping: Dict[int, str]):
        """Установить маппинг для редактирования"""
        self.class_mapping = mapping
        self._load_mapping()



try:
    from PyQt6.QtWidgets import (
        QWidget, QVBoxLayout, QHBoxLayout, QToolBar, QLabel, QFormLayout,
        QGroupBox, QListWidget, QListWidgetItem, QPushButton, QCheckBox, QMessageBox
    )
    from PyQt6.QtGui import QIcon, QAction
    from PyQt6.QtCore import pyqtSignal, pyqtSlot, Qt
    pyqt_version = 6
except ImportError:
    from PyQt5.QtWidgets import (
        QWidget, QVBoxLayout, QHBoxLayout, QToolBar, QAction, QLabel, QFormLayout,
        QGroupBox, QListWidget, QListWidgetItem, QPushButton, QCheckBox, QMessageBox
    )
    from PyQt5.QtGui import QIcon
    from PyQt5.QtCore import pyqtSignal, pyqtSlot, Qt
    pyqt_version = 5

import os
from typing import List

from ..core.logger import get_module_logger
from ..utils import utils as utils_utils

# Используем существующую реализацию холста и логики ROI из диалога
from .roi_core import ROIGraphicsView, ROICoordsDialog


class ROIEditorWindow(QWidget):
    """Немодальное окно для редактирования ROI (Region of Interest)."""

    roi_updated = pyqtSignal(list)  # Сигнал с обновленными ROI (список словарей {'coords': [x1,y1,x2,y2]})
    roi_editor_closed = pyqtSignal(list, int, int, bool)  # rois_xyxy, source_id, detector_index, accepted

    def __init__(self, parent=None):
        super().__init__(parent)
        self.logger = get_module_logger("roi_editor_window")
        self.params = {}

        self.setWindowTitle("ROI Editor")
        self.resize(1200, 800)
        
        # Устанавливаем флаги для независимого окна
        self.setWindowFlags(Qt.WindowType.Window)
        self.setAttribute(Qt.WidgetAttribute.WA_DeleteOnClose, True)

        self.current_source_id = None
        self.current_detector_index = None
        self.saved_rois_data = None
        self.has_unsaved_changes = False
        self._first_show_done = False

        self._create_actions()
        self._create_toolbar()
        self._setup_ui()
    
    def set_params(self, params):
        """Установить параметры (вызывается после controller.init())"""
        self.params = params

    # UI
    def _setup_ui(self):
        layout = QVBoxLayout()
        layout.addWidget(self.drawing_toolbar)

        main_layout = QHBoxLayout()

        # Левая панель - холст
        self.roi_canvas = ROIGraphicsView()
        self.roi_canvas.roi_selected.connect(self._on_roi_selected)
        self.roi_canvas.roi_added.connect(self._on_roi_added)
        self.roi_canvas.roi_removed.connect(self._on_roi_removed)
        self.roi_canvas.roi_updated.connect(self._on_roi_updated)
        main_layout.addWidget(self.roi_canvas, 4)

        # Правая панель - управление
        right_panel = QVBoxLayout()

        #drawing_group = QGroupBox("Drawing Settings")
        #drawing_layout = QFormLayout(drawing_group)
        #right_panel.addWidget(drawing_group)

        roi_group = QGroupBox("ROI Controls")
        roi_layout = QVBoxLayout(roi_group)
        help_label = QLabel("Instructions:\n• Click and drag to create an ROI\n• Click an ROI to select it\n• Select an ROI in the list to edit it")
        help_label.setWordWrap(True)
        roi_layout.addWidget(help_label)

        self.roi_list = QListWidget()
        self.roi_list.setMaximumHeight(120)
        self.roi_list.itemSelectionChanged.connect(self._on_roi_selection_changed)
        roi_layout.addWidget(QLabel("ROI List:"))
        roi_layout.addWidget(self.roi_list)

        roi_buttons = QHBoxLayout()
        self.modify_roi_btn = QPushButton("Edit")
        self.modify_roi_btn.clicked.connect(self._modify_roi)
        self.modify_roi_btn.setEnabled(False)
        roi_buttons.addWidget(self.modify_roi_btn)

        self.delete_roi_btn = QPushButton("Delete")
        self.delete_roi_btn.clicked.connect(self._delete_roi_mode)
        self.delete_roi_btn.setEnabled(False)
        roi_buttons.addWidget(self.delete_roi_btn)
        roi_layout.addLayout(roi_buttons)

        right_panel.addWidget(roi_group)

        settings_group = QGroupBox("Settings")
        settings_layout = QFormLayout(settings_group)
        self.show_numbers_check = QCheckBox("Show numbers")
        self.show_numbers_check.setChecked(True)
        settings_layout.addRow("", self.show_numbers_check)
        right_panel.addWidget(settings_group)

        right_panel.addStretch()
        main_layout.addLayout(right_panel, 1)

        layout.addLayout(main_layout)
        self.setLayout(layout)

    def _create_actions(self):
        # Иконки (пытаемся найти в нескольких локациях)
        def _get_icon(name: str) -> QIcon:
            try:
                base_root = utils_utils.get_project_root()
            except Exception:
                base_root = os.path.dirname(__file__)
            candidates = [
                os.path.join(base_root, 'icons', name),
                os.path.join(os.path.dirname(base_root), 'icons', name),
                os.path.join(os.path.dirname(os.path.dirname(base_root)), 'icons', name),
            ]
            for p in candidates:
                if os.path.exists(p):
                    return QIcon(p)
            return QIcon()

        draw_icon = _get_icon('roi_draw.svg')
        delete_icon = _get_icon('roi_delete.svg')

        self.action_draw_roi = QAction('&Draw ROI', self)
        self.action_draw_roi.setIcon(draw_icon)
        self.action_draw_roi.triggered.connect(self._draw_roi_mode)

        self.action_delete_roi = QAction('&Delete ROI', self)
        self.action_delete_roi.setIcon(delete_icon)
        self.action_delete_roi.triggered.connect(self._delete_roi_mode)

        self.action_clear_all = QAction('&Clear All', self)
        self.action_clear_all.triggered.connect(self._clear_rois)

        self.action_show_all = QAction('&Show All', self)
        self.action_show_all.triggered.connect(self._show_all_rois)

    def _create_toolbar(self):
        self.drawing_toolbar = QToolBar('ROI Tools', self)
        self.drawing_toolbar.addAction(self.action_draw_roi)
        self.drawing_toolbar.addAction(self.action_delete_roi)
        self.drawing_toolbar.addSeparator()
        self.drawing_toolbar.addAction(self.action_clear_all)
        self.drawing_toolbar.addAction(self.action_show_all)

    def showEvent(self, event):
        super().showEvent(event)
        if not self._first_show_done:
            try:
                self.showMaximized()
            except Exception:
                pass
            self._first_show_done = True

    # Public API
    def set_cv_image(self, source_id, cv_image):
        # Импорты зависят от версии PyQt
        import cv2
        if pyqt_version == 6:
            from PyQt6.QtGui import QImage, QPixmap
        else:
            from PyQt5.QtGui import QImage, QPixmap

        # Конвертация BGR->RGB и в QPixmap
        rgb_image = cv2.cvtColor(cv_image, cv2.COLOR_BGR2RGB)
        h, w = rgb_image.shape[:2]
        bytes_per_line = 3 * w
        qt_format = QImage.Format.Format_RGB888 if pyqt_version == 6 else QImage.Format_RGB888
        qimg = QImage(rgb_image.data, w, h, bytes_per_line, qt_format)
        pixmap = QPixmap.fromImage(qimg)

        # Установка изображения в холст
        self.roi_canvas.add_pixmap(pixmap)

        # Сохраняем текущий источник
        self.current_source_id = source_id
        # Устанавливаем коэффициенты преобразования источник→отображение (равны 1.0 при совпадении размеров)
        try:
            src_h, src_w = cv_image.shape[:2]
            disp_w, disp_h = pixmap.width(), pixmap.height()
            scale_x = disp_w / src_w if src_w else 1.0
            scale_y = disp_h / src_h if src_h else 1.0
            self.roi_canvas.source_to_display_scale = {
                'scale_x': scale_x,
                'scale_y': scale_y,
                'source_width': src_w,
                'source_height': src_h,
                'display_width': disp_w,
                'display_height': disp_h
            }
        except Exception:
            pass
        # Форсируем репейнт и подгонку
        try:
            self.roi_canvas.scene.update()
            self.roi_canvas.update()
            self.roi_canvas._delayed_fit_in_view()
        except Exception:
            pass

    def set_context(self, source_id: int, detector_index: int):
        self.current_source_id = source_id
        self.current_detector_index = detector_index
        # Новый контекст — сбрасываем флаги и снимок исходных ROI
        self.has_unsaved_changes = False
        self.saved_rois_data = None

    def set_rois_from_detector(self, rois_xywh: list):
        """Принять ROI из детектора (формат [x,y,w,h]) и установить в canvas (xyxy)."""
        try:
            self.logger.info(f"set_rois_from_detector called with {len(rois_xywh)} ROI")
            self.roi_canvas.clear_rois()
            converted = []
            for item in rois_xywh:
                if len(item) == 4:
                    x, y, w, h = [int(v) for v in item]
                    if w <= 0 or h <= 0:
                        continue
                    # xyxy с правой/нижней границей включительно
                    x2 = x + w - 1
                    y2 = y + h - 1
                    converted.append([x, y, x2, y2])
            # Установим напрямую в roi_data и отрисуем
            self.roi_canvas.roi_data = [{"coords": coords, "color": (255, 0, 0)} for coords in converted]
            self.logger.info(f"Set roi_data with {len(self.roi_canvas.roi_data)} entries")
            setattr(self.roi_canvas, '_loading_from_config', True)
            for i, entry in enumerate(self.roi_canvas.roi_data):
                self.logger.info(f"Adding ROI {i+1}/{len(self.roi_canvas.roi_data)}: {entry['coords']}")
                # Временно отключаем обновление сцены для каждого ROI
                roi_item = self.roi_canvas._create_roi_item(entry["coords"], entry.get("color", (255, 0, 0)))
                if roi_item:
                    self.roi_canvas.rois.append(roi_item)
                self.logger.info(f"ROI {i+1} added successfully")
            setattr(self.roi_canvas, '_loading_from_config', False)
            # Обновляем сцену один раз в конце
            self.roi_canvas.scene.update()
            self.logger.info("Finished adding ROI to canvas")
            # Обновляем список ROI и перерисовываем сцену
            self._update_roi_list()
            self.logger.info("Updated ROI list")
            # Принудительно обновляем canvas
            self.roi_canvas.scene.update()
            self.roi_canvas.update()
            self.roi_canvas.repaint()
            self.logger.info("Updated canvas scene")
            if self.roi_canvas.roi_data:
                self.roi_canvas.ensure_rois_visible()
                self.logger.info("Ensured ROI visibility")
            # Принудительно обновляем все окно
            self.update()
            self.repaint()
            # Сохраняем снимок исходного состояния для корректной проверки изменений
            try:
                self.saved_rois_data = [entry.get("coords", []) for entry in (self.roi_canvas.get_rois() or [])]
            except Exception:
                self.saved_rois_data = [entry.get("coords", []) for entry in self.roi_canvas.roi_data]
            self.has_unsaved_changes = False
        except Exception:
            pass

    def set_rois_from_config(self, params, source_id):
        """Совместимый слой: делегируем на диалоговую логику, но без диалога."""
        if not params:
            return
        try:
            # Воспользуемся теми же методами, что и в диалоге
            # 1) Загрузим ROI
            rois_data = ROIEditorWindow._load_rois_static(params, source_id, self.roi_canvas)

            # 2) Очистим и применим
            self.roi_canvas.clear_rois()
            self.roi_canvas.roi_data = rois_data.copy()
            setattr(self.roi_canvas, '_loading_from_config', True)
            for roi_data in rois_data:
                coords = roi_data.get('coords', [])
                color = roi_data.get('color', (255, 0, 0))
                if len(coords) == 4:
                    self.roi_canvas.add_roi_direct(coords, color)
            setattr(self.roi_canvas, '_loading_from_config', False)
            self._update_roi_list()
            self.roi_canvas.scene.update()
            self.roi_canvas.update()
            if len(self.roi_canvas.roi_data) > 0:
                self.roi_canvas.ensure_rois_visible()
            # Сохраняем снимок исходного состояния для корректной проверки изменений
            try:
                self.saved_rois_data = [entry.get("coords", []) for entry in (self.roi_canvas.get_rois() or [])]
            except Exception:
                self.saved_rois_data = [entry.get("coords", []) for entry in self.roi_canvas.roi_data]
            self.has_unsaved_changes = False
        except Exception as e:
            self.logger.error(f"Error setting ROI from config: {e}")

    @staticmethod
    def _load_rois_static(params, source_id, roi_canvas):
        """Вынесенная часть из логики ROIEditorDialog.load_rois_from_config."""
        try:
            rois_data = []
            detectors = []
            if 'detectors' in params:
                detectors = params['detectors']
            elif 'pipeline' in params and isinstance(params['pipeline'], dict) and 'detectors' in params['pipeline']:
                detectors = params['pipeline']['detectors']
            elif 'pipeline' in params and isinstance(params['pipeline'], list):
                detectors = params['pipeline']
            else:
                return rois_data

            for detector in detectors:
                detector_source_ids = detector.get('source_ids', [])
                if source_id in detector_source_ids:
                    roi_list = detector.get('roi', [])
                    if roi_list and len(roi_list) > 0:
                        source_rois = roi_list[0] if roi_list else []
                        for roi_coords in source_rois:
                            if len(roi_coords) == 4:
                                x, y, width, height = roi_coords
                                x1, y1 = x, y
                                x2, y2 = x + width, y + height

                                if roi_canvas and roi_canvas.original_size:
                                    current_img_width, current_img_height = roi_canvas.original_size
                                    max_x = max(x1, x2)
                                    max_y = max(y1, y2)
                                    if max_x > current_img_width or max_y > current_img_height:
                                        original_img_width = max_x
                                        original_img_height = max_y
                                        scale_x = current_img_width / original_img_width
                                        scale_y = current_img_height / original_img_height
                                        if roi_canvas.source_to_display_scale is None:
                                            roi_canvas.source_to_display_scale = {
                                                'scale_x': scale_x,
                                                'scale_y': scale_y,
                                                'source_width': original_img_width,
                                                'source_height': original_img_height,
                                                'display_width': current_img_width,
                                                'display_height': current_img_height
                                            }
                                        rois_data.append({
                                            'coords': [x1, y1, x2, y2],
                                            'color': (255, 0, 0)
                                        })
                                    else:
                                        rois_data.append({
                                            'coords': [x1, y1, x2, y2],
                                            'color': (255, 0, 0)
                                        })
                                else:
                                    rois_data.append({'coords': [x1, y1, x2, y2], 'color': (255, 0, 0)})
            return rois_data
        except Exception:
            return []

    # Toolbar actions
    @pyqtSlot()
    def _draw_roi_mode(self):
        # В текущей реализации рисование включается нажатием ЛКМ на сцене,
        # поэтому просто показываем подсказку.
        QMessageBox.information(self, "Drawing Mode", "Click and drag over the image to create an ROI.")

    @pyqtSlot()
    def _delete_roi_mode(self):
        current_item = self.roi_list.currentItem()
        if current_item is None:
            QMessageBox.information(self, "Delete ROI", "Select an ROI in the list on the right.")
            return
        index = self.roi_list.row(current_item)
        self.roi_canvas.remove_roi(index)
        self._update_roi_list()
        self._emit_rois_updated()

    @pyqtSlot()
    def _clear_rois(self):
        self.roi_canvas.clear_rois()
        self._update_roi_list()
        self._emit_rois_updated()

    @pyqtSlot()
    def _show_all_rois(self):
        self.roi_canvas.ensure_rois_visible()

    # Handlers / helpers
    def _on_roi_selected(self, roi_data):
        try:
            index = self.roi_canvas.roi_data.index(roi_data)
            self.roi_list.setCurrentRow(index)
            self.modify_roi_btn.setEnabled(True)
            self.delete_roi_btn.setEnabled(True)
        except ValueError:
            pass

    def _on_roi_added(self, coords):
        # Игнорируем события, возникающие при загрузке из конфигурации
        if getattr(self.roi_canvas, '_loading_from_config', False):
            return
        self._update_roi_list()
        self._emit_rois_updated()
        self.has_unsaved_changes = True

    def _on_roi_removed(self, index):
        if getattr(self.roi_canvas, '_loading_from_config', False):
            return
        self._update_roi_list()
        self.modify_roi_btn.setEnabled(False)
        self.delete_roi_btn.setEnabled(False)
        self._emit_rois_updated()
        self.has_unsaved_changes = True

    def _on_roi_updated(self, index, coords):
        if getattr(self.roi_canvas, '_loading_from_config', False):
            return
        self._update_roi_list()
        self._emit_rois_updated()
        self.has_unsaved_changes = True

    def _on_roi_selection_changed(self):
        current_item = self.roi_list.currentItem()
        if current_item:
            self.modify_roi_btn.setEnabled(True)
            self.delete_roi_btn.setEnabled(True)
            current_index = self.roi_list.row(current_item)
            self.roi_canvas.select_roi_by_index(current_index)
        else:
            self.modify_roi_btn.setEnabled(False)
            self.delete_roi_btn.setEnabled(False)
            if self.roi_canvas.selected_roi:
                self.roi_canvas.selected_roi = None

    def _modify_roi(self):
        current_item = self.roi_list.currentItem()
        if not current_item:
            return
        current_index = self.roi_list.row(current_item)
        roi_data = self.roi_canvas.roi_data[current_index]
        coords = roi_data["coords"]
        try:
            dialog = ROICoordsDialog(self, coords)
            result = dialog.exec()
        except Exception as e:
            self.logger.error(f"Error opening ROICoordsDialog: {e}")
            return
        # Универсальная проверка результата диалога для PyQt5/6
        if hasattr(type(dialog), 'DialogCode'):
            accepted_code = type(dialog).DialogCode.Accepted
        elif hasattr(dialog, 'Accepted'):
            accepted_code = dialog.Accepted
        else:
            accepted_code = 1
        if result == accepted_code:
            new_coords = dialog.get_coords()
            if new_coords:
                self.roi_canvas.update_roi(current_index, new_coords)
                self._update_roi_list()
                self._emit_rois_updated()

    def _update_roi_list(self):
        self.roi_list.clear()
        for i, roi in enumerate(self.roi_canvas.roi_data):
            coords = roi.get('coords', [0, 0, 0, 0])
            item = QListWidgetItem(f"ROI_{i+1}: [{coords[0]}, {coords[1]}, {coords[2]}, {coords[3]}]")
            self.roi_list.addItem(item)

    def _emit_rois_updated(self):
        rois = self.roi_canvas.get_rois()
        self.roi_updated.emit(rois)

    # Совместимость: некоторые старые вызовы могли использовать _delete_roi
    def _delete_roi(self):
        self._delete_roi_mode()

    def closeEvent(self, event):
        # Определяем, были ли реальные изменения списка/координат ROI по сравнению с исходным снимком
        def _snapshot_current_rois():
            try:
                return [entry.get("coords", []) for entry in self.roi_canvas.get_rois()]
            except Exception:
                return []

        has_real_changes = self.has_unsaved_changes

        if has_real_changes:
            try:
                if pyqt_version == 6:
                    from PyQt6.QtWidgets import QMessageBox as _QMB
                    StdBtn = _QMB.StandardButton
                    buttons = StdBtn.Yes | StdBtn.No | StdBtn.Cancel
                else:
                    from PyQt5.QtWidgets import QMessageBox as _QMB
                    StdBtn = _QMB
                    buttons = _QMB.Yes | _QMB.No | _QMB.Cancel
            except Exception:
                from PyQt5.QtWidgets import QMessageBox as _QMB
                StdBtn = _QMB
                buttons = _QMB.Yes | _QMB.No | _QMB.Cancel
            mb = _QMB()
            mb.setWindowTitle("Save changes?")
            mb.setText("ROIs have changed. Save changes?")
            mb.setStandardButtons(buttons)
            res = mb.exec()
            yes = StdBtn.Yes
            no = StdBtn.No
            cancel = StdBtn.Cancel
            if res == yes:
                rois = self.roi_canvas.get_rois()
                rois_xyxy = [entry["coords"] for entry in rois]
                self.roi_editor_closed.emit(rois_xyxy, int(self.current_source_id or 0), int(self.current_detector_index or -1), True)
                event.accept()
            elif res == no:
                self.roi_editor_closed.emit([], int(self.current_source_id or 0), int(self.current_detector_index or -1), False)
                event.accept()
            else:
                event.ignore()
                return
        else:
            rois = self.roi_canvas.get_rois()
            rois_xyxy = [entry["coords"] for entry in rois]
            self.roi_editor_closed.emit(rois_xyxy, int(self.current_source_id or 0), int(self.current_detector_index or -1), False)
            event.accept()





from fastapi import APIRouter, HTTPException
from pydantic import BaseModel, Field
from typing import List, Dict, Optional
from pathlib import Path
import json

""" Module for managing configuration files via the API"""

router = APIRouter(prefix="/api/v1/configs", tags=["configs"])


class ConfigUpsert(BaseModel):
    name: str = Field(..., description="The name of the configuration file, for example single_video.json")
    body: dict = Field(..., description="JSON configuration content")


# Model for opening (running) a config
class ConfigRunCreate(BaseModel):
    name: Optional[str] = Field(None, description="Human-readable name (auto-generated if not provided)")
    config_name: Optional[str] = Field(None, description="Configuration name from the configs folder")
    config_body: Optional[dict] = Field(None, description="Configuration body, if file name not used")


@router.get("/list")
async def list_configs() -> List[str]:
    configs_dir = Path("configs")
    if not configs_dir.exists():
        return []
    return sorted([p.name for p in configs_dir.glob("*.json")])


@router.get("/{name}")
# Passing the file name to the url is not very good practice
async def get_config(name: str) -> dict:
    path = Path("configs") / name
    if not path.exists() or not path.is_file():
        raise HTTPException(status_code=404, detail="Config not found")
    try:
        with open(path, "r", encoding="utf-8") as f:
            return json.load(f)
    except Exception as e:
        raise HTTPException(status_code=500, detail=f"Failed to read config: {e}")


# create config file
@router.post("/create")
async def create_config(payload: ConfigUpsert):
    name = Path(payload.name).name
    if not name.endswith(".json"):
        raise HTTPException(status_code=400, detail="Config name must end with .json")
    cfg_dir = Path("configs")
    cfg_dir.mkdir(parents=True, exist_ok=True)
    path = cfg_dir / name
    if path.exists():
        raise HTTPException(status_code=409, detail="Config already exists")
    try:
        with open(path, "w", encoding="utf-8") as f:
            json.dump(payload.body, f, ensure_ascii=False, indent=2)
        return {"name": name, "status": "created"}
    except Exception as e:
        raise HTTPException(status_code=500, detail=f"Failed to create config: {e}")


@router.put("/update")
# Passing the file name to the uri is not very good practice
async def update_config(payload: ConfigUpsert):
    name_config = payload.name
    target = Path("configs") / Path(name_config).name
    if not target.exists():
        raise HTTPException(status_code=404, detail="Config not found")
    try:
        with open(target, "w", encoding="utf-8") as f:
            json.dump(payload.body, f, ensure_ascii=False, indent=2)
        return {"name": target.name, "status": "updated"}
    except Exception as e:
        raise HTTPException(status_code=500, detail=f"Failed to update config: {e}")


# Open config to run (create run) — placed after Update for docs order
@router.post("/open", summary="Open Config to run")
async def create_config_run(payload: ConfigRunCreate) -> Dict:
    data = payload.model_dump()
    rid = len(get_config_run_manager().list()) + 1
    try:
        return get_config_run_manager().create(
            rid,
            data.get("name"),
            config_name=data.get("config_name"),
            config_body=data.get("config_body"),
        )
    except FileNotFoundError:
        raise HTTPException(status_code=404, detail="Config file not found")
    except ValueError as e:
        raise HTTPException(status_code=400, detail=str(e))


@router.delete("/{name}")
# Passing the file name to the uri is not very good practice
async def delete_config(name: str):
    target = Path("configs") / Path(name).name
    if not target.exists():
        raise HTTPException(status_code=404, detail="Config not found")
    try:
        target.unlink()
        return {"name": target.name, "status": "deleted"}
    except Exception as e:
        raise HTTPException(status_code=500, detail=f"Failed to delete config: {e}")

from evileye.api.core.config_run_access import get_config_run_manager


@router.get("/runs/list")
async def list_config_runs() -> Dict[int, Dict]:
    return get_config_run_manager().list()


 


@router.get("/runs/{rid}/status")
async def get_config_run(rid: int) -> Dict:
    try:
        return get_config_run_manager().describe(rid)
    except KeyError:
        raise HTTPException(status_code=404, detail="Config run not found")


@router.post("/runs/{rid}/start")
async def start_config_run(rid: int) -> Dict:
    try:
        return get_config_run_manager().start(rid)
    except KeyError:
        raise HTTPException(status_code=404, detail="Config run not found")
    except Exception as e:
        raise HTTPException(status_code=500, detail=str(e))


@router.post("/runs/{rid}/stop")
async def stop_config_run(rid: int) -> Dict:
    try:
        return get_config_run_manager().stop(rid)
    except KeyError:
        raise HTTPException(status_code=404, detail="Config run not found")
    except Exception as e:
        raise HTTPException(status_code=500, detail=str(e))


@router.delete("/runs/{rid}/delete")
async def delete_config_run(rid: int) -> Dict:
    try:
        return get_config_run_manager().delete(rid)
    except KeyError:
        raise HTTPException(status_code=404, detail="Config run not found")
    except RuntimeError as e:
        raise HTTPException(status_code=409, detail=str(e))



from abc import ABC, abstractmethod
from ..core.base_class import EvilEyeBase
import threading
from queue import Queue


class DatabaseControllerBase(EvilEyeBase):
    def __init__(self, controller_type):
        super().__init__()
        self.host_name = "localhost"
        self.database_name = ""
        self.user_name = ""
        self.password = ""
        self.controller_type = controller_type
        self.logger.info(f"Controller type: {controller_type}")
        self.run_flag = False
        if self.controller_type == 'Writer':
            self.query_thread = threading.Thread(target=self._insert_impl)
        else:
            self.query_thread = None
        self.queue_in = Queue()
        self.queue_out = Queue()

    def connect(self):
        if self.get_init_flag():
            return self.connect_impl()
        else:
            raise Exception('init function has not been called')

    def disconnect(self):
        if self.get_init_flag():
            return self.disconnect_impl()
        else:
            raise Exception('init function has not been called')

    def get(self):
        if self.queue_out.empty():
            return None
        else:
            return self.queue_out.get()

    def query(self, query_string, data=None):
        if self.get_init_flag():
            return self.query_impl(query_string, data)
        else:
            raise Exception('init function has not been called')

    def start(self):
        if self.query_thread:
            self.logger.info('Started writer db')
            self.run_flag = True
            self.query_thread.start()

    def stop(self):
        if self.query_thread:
            self.run_flag = False
            self.queue_in.put((None,))
            if self.query_thread.is_alive():
                self.query_thread.join()
        self.logger.info('DataBase stopped')

    @abstractmethod
    def connect_impl(self):
        pass

    @abstractmethod
    def disconnect_impl(self):
        pass

    @abstractmethod
    def query_impl(self, query_string, data=None):
        pass

    @abstractmethod
    def _insert_impl(self):
        pass



from .processor_base import ProcessorBase
from .frame import Frame


class ProcessorStep(ProcessorBase):
    def __init__(self, processor_name, class_name, num_processors: int, order: int):
        super().__init__(processor_name, class_name, num_processors, order)

    def process(self, input_list=None):
        processing_results = []
        if input_list is not None:
            for input in input_list:
                is_processor_found = False
                if (type(input) == list or type(input) == tuple) and len(input) >= 2:
                    data = input[0]
                    frame = input[1]
                elif type(input) == Frame:
                    frame = input
                    data = None
                else:
                    raise RuntimeError(f"Wrong type for input data in processor: {self.class_name}")

                for processor in self.processors:
                    source_ids = processor.get_source_ids()
                    if frame.source_id in source_ids:
                        processor.put(input)
                        is_processor_found = True

                    if is_processor_found:
                        break

                if not is_processor_found:
                    res = self.dummy_processor.ResultType()
                    if res is not None:
                        if hasattr(res, "source_id"):
                            setattr(res, "source_id", frame.source_id)
                        if hasattr(res, "frame_id"):
                            setattr(res, "frame_id", frame.frame_id)
                        if hasattr(res, "time_stamp"):
                            setattr(res, "time_stamp", frame.time_stamp)
                        if hasattr(res, "generate_from"):
                            res.generate_from(data)

                    processing_results.append([res, frame])

        for processor in self.processors:
            result = processor.get()
            if result:
                processing_results.append(result)

        return processing_results


from threading import Lock

events = dict()
mutex = Lock()


def subscribe(event, subscriber_func):
    with mutex:
        if event not in events:
            events[event] = []
        events[event].append(subscriber_func)


def notify(event, *args, **kwargs):
    with mutex:
        if event not in events:
            return
        functions = events[event]
    for func in functions:
        func(*args, **kwargs)



"""
Tests for GUI modes (headless, hidden, visible).
"""

import unittest
import json
import tempfile
import os
from pathlib import Path

from evileye.gui import GUIMode, determine_gui_mode


class TestGUIModes(unittest.TestCase):
    """Test GUI mode determination."""
    
    def test_headless_mode_from_cli(self):
        """Test headless mode when CLI gui=False."""
        config = {"controller": {}}
        mode = determine_gui_mode(config, cli_gui=False)
        self.assertEqual(mode, GUIMode.HEADLESS)
    
    def test_visible_mode_from_cli(self):
        """Test visible mode when CLI gui=True."""
        config = {"controller": {}}
        mode = determine_gui_mode(config, cli_gui=True)
        self.assertEqual(mode, GUIMode.VISIBLE)
    
    def test_hidden_mode_from_config(self):
        """Test hidden mode from config."""
        config = {"controller": {"gui_mode": "hidden"}}
        mode = determine_gui_mode(config, cli_gui=True)
        self.assertEqual(mode, GUIMode.HIDDEN)
    
    def test_headless_mode_from_config(self):
        """Test headless mode from config."""
        config = {"controller": {"gui_mode": "headless"}}
        mode = determine_gui_mode(config, cli_gui=None)
        self.assertEqual(mode, GUIMode.HEADLESS)
    
    def test_visible_mode_from_config(self):
        """Test visible mode from config."""
        config = {"controller": {"gui_mode": "visible"}}
        mode = determine_gui_mode(config, cli_gui=None)
        self.assertEqual(mode, GUIMode.VISIBLE)
    
    def test_legacy_gui_enabled_false(self):
        """Test legacy config: gui_enabled=False."""
        config = {"controller": {"gui_enabled": False}}
        mode = determine_gui_mode(config, cli_gui=None)
        self.assertEqual(mode, GUIMode.HEADLESS)
    
    def test_legacy_show_main_gui_false(self):
        """Test legacy config: show_main_gui=False."""
        config = {"controller": {"gui_enabled": True, "show_main_gui": False}}
        mode = determine_gui_mode(config, cli_gui=None)
        self.assertEqual(mode, GUIMode.HIDDEN)
    
    def test_legacy_show_main_gui_true(self):
        """Test legacy config: show_main_gui=True."""
        config = {"controller": {"gui_enabled": True, "show_main_gui": True}}
        mode = determine_gui_mode(config, cli_gui=None)
        self.assertEqual(mode, GUIMode.VISIBLE)
    
    def test_default_visible(self):
        """Test default mode is visible."""
        config = {"controller": {}}
        mode = determine_gui_mode(config, cli_gui=None)
        self.assertEqual(mode, GUIMode.VISIBLE)
    
    def test_cli_overrides_config(self):
        """Test CLI argument overrides config."""
        config = {"controller": {"gui_mode": "hidden"}}
        mode = determine_gui_mode(config, cli_gui=False)
        self.assertEqual(mode, GUIMode.HEADLESS)  # CLI should override


class TestGUIManager(unittest.TestCase):
    """Test GUIManager initialization."""
    
    def test_headless_mode_no_qt_app(self):
        """Test that headless mode doesn't create QApplication."""
        from evileye.gui import GUIManager
        
        config = {"controller": {}}
        manager = GUIManager(GUIMode.HEADLESS, config)
        
        # Headless mode should not create QApplication
        self.assertIsNone(manager.qt_app)
        self.assertIsNone(manager.main_window)
    
    def test_flag_file_path_initialization(self):
        """Test that flag file path is initialized in constructor."""
        from evileye.gui import GUIManager
        
        config = {"controller": {"gui_flag_file": "test/.show_gui"}}
        manager = GUIManager(GUIMode.HIDDEN, config)
        
        # Flag file path should be initialized in constructor
        self.assertIsNotNone(manager.flag_file_path)
        self.assertEqual(str(manager.flag_file_path), "test/.show_gui")
    
    def test_default_flag_file_path(self):
        """Test default flag file path."""
        from evileye.gui import GUIManager
        
        config = {"controller": {}}
        manager = GUIManager(GUIMode.HIDDEN, config)
        
        # Default flag file path
        self.assertIsNotNone(manager.flag_file_path)
        self.assertEqual(str(manager.flag_file_path), "EvilEyeData/.show_gui")


class TestVisualizationAdapter(unittest.TestCase):
    """Test VisualizationAdapter."""
    
    def test_adapter_without_visualizer(self):
        """Test adapter without visualizer."""
        from evileye.gui.visualization_adapter import VisualizationAdapter
        
        adapter = VisualizationAdapter()
        self.assertFalse(adapter.is_available())
        self.assertEqual(adapter.get_params(), {})
    
    def test_adapter_set_visualizer(self):
        """Test setting visualizer in adapter."""
        from evileye.gui.visualization_adapter import VisualizationAdapter
        
        adapter = VisualizationAdapter()
        
        # Create mock visualizer
        class MockVisualizer:
            def start(self):
                pass
            def stop(self):
                pass
            def get_params(self):
                return {"test": "value"}
        
        mock_visualizer = MockVisualizer()
        adapter.set_visualizer(mock_visualizer)
        
        self.assertTrue(adapter.is_available())
        self.assertEqual(adapter.get_params(), {"test": "value"})


if __name__ == '__main__':
    unittest.main()



import os
from .object_detection_base import ObjectDetectorBase
from .detection_thread_yolo import DetectionThreadYolo
from ..core.base_class import EvilEyeBase


@EvilEyeBase.register("ObjectDetectorYolo")
class ObjectDetectorYolo(ObjectDetectorBase):
    id_cnt = 0  # Переменная для присвоения каждому детектору своего идентификатора

    def __init__(self):
        super().__init__()
        self.model_name = "models/yolo11n.pt"

    def init_impl(self):
        super().init_impl()
        self.detection_threads = []
        inf_params = {"show": self.params.get('show', False), 'conf': self.params.get('conf', 0.25),
                      'save': self.params.get('save', False), "imgsz": self.params.get('inference_size', 640),
                      "device": self.params.get('device', None)}

        for i in range(self.num_detection_threads):
            # Resolve relative model path to current working directory for access
            model_path = self.model_name
            if not os.path.isabs(model_path):
                model_path = os.path.join(os.getcwd(), model_path)
            
            logger_name = f"det{i}"
            thread = DetectionThreadYolo(model_path, self.stride, self.classes, self.source_ids, self.roi, inf_params,
                                         self.queue_out, logger_name=logger_name, parent_logger=self.logger)
            thread.start()
            self.detection_threads.append(thread)
        return True

    def reset_impl(self):
        super().reset_impl()

    def set_params_impl(self):
        super().set_params_impl()
        self.model_name = self.params.get('model', self.model_name)

    def get_params_impl(self):
        params = super().get_params_impl()
        params['model'] = self.model_name
        return params

    def get_debug_info(self, debug_info: dict):
        super().get_debug_info(debug_info)
        debug_info['model_name'] = self.model_name

    def default(self):
        super().default()
        self.model_name = None
        self.params.clear()



"""
Извлечение метаданных событий для отображения в журналах
"""

from typing import Optional, Tuple, List, Dict


class EventMetadataExtractor:
    """Класс для извлечения метаданных событий из различных источников"""
    
    @staticmethod
    def get_bbox_and_zone(event_data: dict, is_lost: bool = False) -> Tuple[Optional[List[float]], Optional[List]]:
        """Извлечь bounding box и zone coordinates из данных события
        
        Учитывает различные форматы данных:
        - Для событий из БД: могут быть поля box_entered/box_left или bounding_box/lost_bounding_box
        - Для событий из JSON: могут быть различные поля в зависимости от типа события
        
        Args:
            event_data: Словарь с данными события
            is_lost: True если нужно извлечь данные для lost события
            
        Returns:
            Кортеж (bbox, zone_coords), где:
            - bbox: список [x1, y1, x2, y2] или [x, y, w, h] в зависимости от формата
            - zone_coords: список координат зоны [(x1, y1), (x2, y2), ...] или None
        """
        if not event_data:
            return None, None
        
        event_type = event_data.get('event_type', '')
        box = None
        zone_coords = None
        
        # Обработать разные типы событий
        if event_type in ('zone_entered', 'zone_left'):
            # ZoneEvent: использовать box_entered/box_left или bounding_box/lost_bounding_box
            if is_lost:
                # Попробовать разные варианты полей
                box = (event_data.get('box_left') or 
                      event_data.get('lost_bounding_box') or 
                      event_data.get('bounding_box'))
            else:
                box = (event_data.get('box_entered') or 
                      event_data.get('bounding_box'))
            
            zone_coords = event_data.get('zone_coords')
            
        elif event_type in ('attr_found', 'attr_lost'):
            # AttributeEvent: использовать box_found/box_finished или bounding_box/lost_bounding_box
            if is_lost:
                box = (event_data.get('box_finished') or 
                      event_data.get('lost_bounding_box') or 
                      event_data.get('bounding_box'))
            else:
                box = (event_data.get('box_found') or 
                      event_data.get('bounding_box'))
            
        elif event_type in ('found', 'lost', 'ObjectEvent'):
            # ObjectEvent: использовать bounding_box
            box = event_data.get('bounding_box') or event_data.get('box')
            
        else:
            # Fallback: попробовать общие поля
            if is_lost:
                box = (event_data.get('lost_bounding_box') or 
                      event_data.get('bounding_box') or 
                      event_data.get('box'))
            else:
                box = event_data.get('bounding_box') or event_data.get('box')
            
            zone_coords = event_data.get('zone_coords')
        
        return box, zone_coords
    
    @staticmethod
    def normalize_bbox_for_display(box, img_w: int, img_h: int) -> Optional[List[float]]:
        """Нормализовать bounding box для отображения с учетом размеров изображения
        
        Args:
            box: Bounding box в любом формате (dict, list, tuple)
            img_w: Ширина изображения
            img_h: Высота изображения
            
        Returns:
            Нормализованный список [x1, y1, x2, y2] в диапазоне [0,1] или None
        """
        if not box or img_w <= 0 or img_h <= 0:
            return None
        
        try:
            # Обработать dict формат
            if isinstance(box, dict):
                x = float(box.get('x', 0))
                y = float(box.get('y', 0))
                w = float(box.get('width', 0))
                h = float(box.get('height', 0))
                
                # Если координаты уже нормализованы
                if max(x, y, w, h) <= 1.0:
                    return [x, y, x + w, y + h]
                else:
                    # Нормализовать
                    return [x / img_w, y / img_h, (x + w) / img_w, (y + h) / img_h]
            
            # Обработать list/tuple формат
            elif isinstance(box, (list, tuple)) and len(box) == 4:
                a, b, c, d = [float(x) for x in box]
                
                # Если уже нормализовано [x1, y1, x2, y2]
                if max(a, b, c, d) <= 1.0:
                    return [a, b, c, d]
                else:
                    # Предполагаем [x, y, w, h] в пикселях
                    return [a / img_w, b / img_h, (a + c) / img_w, (b + d) / img_h]
        except Exception:
            pass
        
        return None
    
    @staticmethod
    def normalize_zone_coords(zone_coords, img_w: int, img_h: int) -> Optional[List[Tuple[float, float]]]:
        """Нормализовать координаты зоны для отображения
        
        Args:
            zone_coords: Координаты зоны (список кортежей или строка)
            img_w: Ширина изображения
            img_h: Высота изображения
            
        Returns:
            Нормализованный список кортежей [(x1, y1), (x2, y2), ...] или None
        """
        if not zone_coords or img_w <= 0 or img_h <= 0:
            return None
        
        try:
            # Если строка, распарсить
            if isinstance(zone_coords, str):
                zone_coords = EventMetadataExtractor._parse_zone_coords_string(zone_coords)
            
            # Если список кортежей
            if isinstance(zone_coords, (list, tuple)):
                normalized = []
                for coord in zone_coords:
                    if isinstance(coord, (list, tuple)) and len(coord) == 2:
                        px, py = float(coord[0]), float(coord[1])
                        
                        # Если координаты уже нормализованы
                        if px <= 1.0 and py <= 1.0:
                            normalized.append((px, py))
                        else:
                            # Нормализовать
                            normalized.append((px / img_w, py / img_h))
                
                return normalized if normalized else None
        except Exception:
            pass
        
        return None
    
    @staticmethod
    def _parse_zone_coords_string(zone_str: str) -> Optional[List[Tuple[float, float]]]:
        """Распарсить строку координат зоны из БД формата"""
        try:
            s = zone_str.strip().strip('{}')
            points_str = [p.strip('{} ') for p in s.split('},')]
            coords = []
            for ps in points_str:
                parts = [pp.strip() for pp in ps.split(',') if pp.strip()]
                if len(parts) == 2:
                    coords.append((float(parts[0]), float(parts[1])))
            return coords if coords else None
        except Exception:
            return None



from queue import Queue
import threading
from .detection_thread_base import DetectionThreadBase

# Import utils later to avoid circular imports
utils = None

def get_utils():
    global utils
    if utils is None:
        from evileye.utils import utils as utils_module
        utils = utils_module
    return utils


class DetectionThreadYoloMp(DetectionThreadBase):
    id_cnt = 0  # Переменная для присвоения каждому детектору своего идентификатора

    def __init__(self, model_name: str, stride: int, classes: list, source_ids: list, roi: list, inf_params: dict, queue_out: Queue):
        # Import here to avoid circular imports
        from evileye.core.mp_control import MpControl
        from .mp_worker_yolo import MpWorkerYolo
        
        self.mp_control = MpControl(max_input_size=len(roi))
        self.mp_worker = self.mp_control.add_worker(MpWorkerYolo)
        self.model_name = model_name
        self.model = None
        super().__init__(stride, classes, source_ids, roi, inf_params, queue_out)
        self.mp_worker.set_params(self.model_name, self.classes, self.inf_params)
        self.mp_control.start()

    def predict(self, images: list):
        self.mp_control.put(images)
        res = self.mp_control.get()
        return res

    def get_bboxes(self, result, roi):
        bboxes_coords = []
        confidences = []
        ids = []
        boxes = result.boxes.numpy()
        coords = boxes.xyxy
        confs = boxes.conf
        class_ids = boxes.cls
        for coord, class_id, conf in zip(coords, class_ids, confs):
            if int(class_id) not in self.classes:
                continue
            utils_module = get_utils()
            abs_coords = utils_module.roi_to_image(coord, roi[1][0], roi[1][1])  # Получаем координаты рамки в СК всего изображения
            bboxes_coords.append(abs_coords)
            confidences.append(conf)
            ids.append(class_id)
        return bboxes_coords, confidences, ids
